/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package core;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

/**
 * Class is usually only used once or on updates replaced by another instance. It holds a list
 * of Titles, the current index, ...
 */
public class Playlist {
	private LinkedList pl;
	private int position = 0;
	
	public Playlist(){
		pl = new LinkedList();
	}
	
	/**
	 * Create a new Playlist and put List l as start Elements. But be careful that this list
	 * only contains Titles!
	 * 
	 * @param l
	 */
	public Playlist(List l){
		pl = new LinkedList();
		add(l);
	}
	
	/**
	 * Add l to the current playlist, be careful that this list contains only Titles
	 * @param l
	 */
	public void add(List l){
		for (ListIterator it = l.listIterator(); it.hasNext();){
			Object o = it.next();
			if (o instanceof Title)
				add((Title)o);
			else if (o instanceof String)
				pl.add(new Title((String)o));
		}
	}
	
	/**
	 * add Entries of Playlist l to the this Playlist
	 * @param l
	 */
	public void mergeList(Playlist l){
		add(l.toList());
	}
	
	/**
	 * @return		Titles of this playlist as List
	 */
	public List toList(){
		return pl;
	}
	
	/**
	 * 
	 * @return		List of ids
	 */
	public List getIDs(){
		List ids = new ArrayList();
		for (int i = 0; i < pl.size(); i++)
			ids.add(((Title)pl.get(i)).getAttribute("id"));
		return ids;
	}
	
	/**
	 * add a Title to this list
	 * @param t
	 */
	public void add(Title t){
		pl.add(t);
	}
	
	public String toString(){
		String output = "";
		int i = 1;
		for (ListIterator it = pl.listIterator(); it.hasNext();){
			Title t = (Title)it.next();
			output += i++ + "\t" + t + "\n";
		}
		return output;
	}
	
	public ListIterator listIterator(){
		return pl.listIterator();
	}
	
	public Iterator Iterator(){
		return pl.iterator();
	}
	
	public void updateTitleAtIndex(int index, Title t){
		pl.set(index, t);
	}
	
	/**
	 * @param t
	 * @return		index of Title t
	 */
	public int getIndex(Title t){
		return pl.indexOf(t);
	}
	
	/**
	 * remove Title from playlist
	 * @param t
	 * @return		removed Title
	 */
	public Title removeTitle(Title t){
		if (pl.contains(t)){
			Title ti = (Title)pl.get(getIndex(t));
			pl.remove(t);
			return ti;
		}
		return null;
	}
	
	/**
	 * Check if this playlist contains the given id
	 * @param id
	 * @return		true if yes, false otherwise
	 */
	public List indecesOfID(int id){
		ArrayList indeces = new ArrayList();
		int i = 0;
		for (ListIterator it = pl.listIterator(); it.hasNext();){
			if (Integer.parseInt(((Title)it.next()).getAttribute("id")) == id) indeces.add(new Integer(i));
			i++;
		}
		return indeces;
	}
	
	/**
	 * @param index
	 * @return		Title on index
	 */
	public Title getTitleForIndex(int index) {
		return (Title)pl.get(index);
	}
	
	/**
	 * 
	 * @return		size of Playlist
	 */
	public int getSize(){
		return pl.size();
	}
	
	/**
	 * 
	 * @return		current position
	 */
	public int getPosition(){
		return position;
	}
	
	/**
	 * @return		current marked Title		
	 */
	public Title getCurrentTitle(){
		try {
			return (Title)pl.get(position);
		}catch (Exception e){
			return new Title();
		}
	}
	
	/**
	 * set a new position
	 * @param pos
	 */
	public void setPosition(int pos){
		if (pos > pl.size()-1)
			position = pl.size()-1;
		else position = pos;
	}
	
	/**
	 * clear playlist
	 *
	 */
	public void clear(){
		pl.clear();
	}
	
	public boolean equals(Object o){
		if (!(o instanceof Playlist)) return false;
		Playlist p = (Playlist)o;
		if (p.getSize() != getSize()) return false;
		for (int i = 0; i < getSize(); i++)
			if (!p.getTitleForIndex(i).equals(getTitleForIndex(i))) return false;
		return true;
	}
}
