/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/
	
package core;

/**
 * ReturnValue is used to get various returnValues from the
 * underlaying daemon which is doing all the work to the Management
 * ReturnValues never reach the UI!
 */

public final class ReturnValue {
	private int type;
	private long threadID;
	private Object value;
	private long stamp;
	
	public static final int ERROR_TYPE = -1;
	public static final int VOID_TYPE = 0;
	public static final int LONG_TYPE = 1;
	public static final int DOUBLE_TYPE = 2;
	public static final int STRING_TYPE = 3;
	public static final int PLAYLIST_TYPE = 4;
	public static final int MEDIALIB_TYPE = 5;
	public static final int MEDIALIB_SEARCH_TYPE = 6;
	public static final int LIST_TYPE = 7;
	public static final int HASHMAP_TYPE = 8;
	public static final int PLAYLISTINFORMATION_TYPE = 9;
	public static final int TITLE_TYPE = 10;
	public static final int ERROR_STATE_TYPE = 51;
	public static final int LOVE_TYPE = 100;
	
	public ReturnValue(long threadID, long time){
		type = VOID_TYPE;
		this.threadID = threadID;
		value = null;
		stamp = time;
	}

	public long getThreadID() {
		return threadID;
	}

	public int getType() {
		return type;
	}

	public Object getValue() {
		return value;
	}
	
	public void setValue(Object value) {
		this.value = value;
	}
	
	public void setType(int type){
		this.type = type;
	}
	
	public String toString(){
		return "[" + getThreadID() + "\t" + value + "\t" + getType() + "]";
	}
	
	public long getTimeStamp(){
		return stamp;
	}
	
	public void setTimeStamp(long time){
		stamp = time;
	}
}
