/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * Copyright Georg Schild (dangertools@gmail.com), Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package core;

import java.util.Observable;
import java.util.Observer;

import org.apache.log4j.Logger;

/**
 * ThreadFactory has to be implemented by the backend. It is used to create one KeepStateThread
 * and mass of Medialib and Controlthreads - or even not, belongs your implementation
 */
public abstract class ThreadFactory extends Observable{
	protected Logger logger;
	protected boolean connected = false;
	protected Management mm;
	
	protected static String defaultSQLString = "";
	
	protected ThreadFactory(Management mm){
		logger = Logger.getLogger(getClass());
		this.mm = mm;
		addObserver(mm);
	}
	
	/**
	 * Return a specific ControlThread if possible, otherwise null should be returned
	 * 
	 * @param ret		ReturnValue the ControlThread holds
	 * @param obs		observer which should be notified 
	 * @return			ControlThread or null
	 */
	public abstract ManagedThread getControlThread(ReturnValue ret, Observer obs);

	/**
	 * Return a specific MedialibThread if possible, otherwise null should be returned
	 * 
	 * @param ret		ReturnValue the MedialibThread holds
	 * @param obs		observer which should be notified 
	 * @return			MedialibThread or null
	 */
	public abstract ManagedThread getMedialibThread(ReturnValue ret, Observer obs);
	
	/**
	 * return always a KeepStateThread, only one should be needed through Execution
	 * @param obs 
	 * 
	 * @return			KeepStateThread
	 */
	public abstract KeepStateThread getKeepStateThread(Observer obs);
	
	/**
	 * You can implement things to spin down connection[s] to the backend
	 */
	public abstract void spinDown();
	
	/**
	 * call things to connect to the backend (if needed)
	 */
	public abstract void connect();
	
	public boolean isConnected(){
		return connected;
	}
}