/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * Copyright Georg Schild (dangertools@gmail.com), Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package core;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Titles are used in some places in the ui, e.g. in Medialib views, current playlist, titleinformation, ...
 */
public class Title implements Comparable{
	private int length;
	private HashMap attributes;
	
	public Title(String url){
		attributes = new HashMap();
		setAttribute("url", url);
	}
	
	public Title(){
		attributes = new HashMap();
		setAttribute("artist", "[Unknown artist]");
		setAttribute("album", "[Unknown album]");
		setAttribute("title", "[Unknown title]");
	}
	
	public void setLength(int s){
		length = s;
	}
	
	public int getLength(){
		return length;
	}
	
	/**
	 * Adds/Sets a key to a specified value. keys are stored and received lowercase
	 * 
	 * @param key
	 * @param value
	 */
	public void setAttribute(String key, String value){
		if (value == null)
			value = "";
		if ( key != null)
			attributes.put(key.toLowerCase(), value);
	}
	
	public String getAttribute(String key){
		Object value = attributes.get(key.toLowerCase());
		if (value == null)
			value = "";
		return ""+value;
	}
	
	public Map getAttributes(){
		return attributes;
	}
	
	public String toString(){
		return getAttribute("title") + "\n\t" +
		getAttribute("album") + "\n\t" + 
		getAttribute("artist") + "\n\t" + 
		getAttribute("url") + "\n";
	}
	
	public boolean equals(Object o){
		if (!(o instanceof Title)) return false;
		Title t = (Title)o;
		if (t.getAttributes().size() != getAttributes().size()) return false;
		Iterator a = t.getAttributes().keySet().iterator();
		while (a.hasNext()){
			String keyA = (String)a.next();
			String valA = t.getAttribute(keyA);
			String valB = getAttribute(keyA);
			if (!valA.equals(valB)) return false;
		}
		return true;
	}

	public int compareTo(Object arg0) {
		if (arg0 instanceof Title){
			Title t = (Title)arg0;
			try {
				return new Integer(getAttribute("id")).compareTo(new Integer(t.getAttribute("id")));
			} catch (Exception e){}
		}
		return 0;
	}
}
