/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package core.xmms2;


import java.util.Observer;

import org.xmms2.xmms2bindings.*;

import core.ControlThread;
import core.KeepStateThread;
import core.ManagedThread;
import core.Management;
import core.ReturnValue;

/**
 * Implementation of a xmms2 specific ControlThread. for Documentation on the various methods
 * have a look at core.ControlThread - noone likes to do things twice ;)
 */

 public final class Xmms2ControlThread extends ManagedThread implements ControlThread {	
	private SWIGTYPE_p_xmmsc_connection_St myConnection;
	private SWIGTYPE_p_xmmsc_result_St result;
	
	protected Xmms2ControlThread(ReturnValue ret, Observer ob, Management mm, SWIGTYPE_p_xmmsc_connection_St connection){
		super(ret, ob, mm);
		myConnection = connection;
	}
	
	public ReturnValue play() {
		logger.debug("play");
		result = Xmmsclient.xmmsc_playback_start(myConnection);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue stop() {
		logger.debug("stop");
		result = Xmmsclient.xmmsc_playback_stop(myConnection);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue pause() {
		if (mm.getPlaybackStatus() == KeepStateThread.PLAYING)
			result = Xmmsclient.xmmsc_playback_pause(myConnection);
		else if (mm.getPlaybackStatus() == KeepStateThread.PAUSED)
			result = Xmmsclient.xmmsc_playback_start(myConnection);
		logger.debug("pause");
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue next() {
		return nextInPlaylistRel(1);
	}
	
	public ReturnValue previous() {
		return nextInPlaylistRel(-1);
	}
	
	public ReturnValue seek(long ms) {
		result = Xmmsclient.xmmsc_playback_seek_ms(myConnection, (int)ms);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue nextInPlaylistAbs(int abs) {
		result = Xmmsclient.xmmsc_playlist_set_next(myConnection, abs);
		Xmmsclient.xmmsc_result_wait(result);
		if (mm.getPlaybackStatus() == KeepStateThread.PLAYING){
			result = Xmmsclient.xmmsc_playback_tickle(myConnection);
			Xmmsclient.xmmsc_result_wait(result);
			return ret;
		}
		return play();
	}
	
	public ReturnValue nextInPlaylistRel(int rel) {
		result = Xmmsclient.xmmsc_playlist_set_next_rel(myConnection, rel);
		Xmmsclient.xmmsc_result_wait(result);
		result = Xmmsclient.xmmsc_playback_tickle(myConnection);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue playlistShuffle() {
		result = Xmmsclient.xmmsc_playlist_shuffle(myConnection);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue playlistClear() {
		result = Xmmsclient.xmmsc_playlist_clear(myConnection);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue playlistSort(String by) {
		result = Xmmsclient.xmmsc_playlist_sort(myConnection, by);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue addTitleToPlaylist(String path) {
		result = Xmmsclient.xmmsc_playlist_add(myConnection, path);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue addTitleToPlaylistIndex(String path, int index) {
		result = Xmmsclient.xmmsc_playlist_insert(myConnection, index, path);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue addTitleToPlaylist(int id) {
		result = Xmmsclient.xmmsc_playlist_add_id(myConnection, id);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}

	public ReturnValue addTitleToPlaylist(int id, int index) {
		result = Xmmsclient.xmmsc_playlist_insert_id(myConnection, index, id);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue removeTitleFromPlaylist(long index) {
		result = Xmmsclient.xmmsc_playlist_remove(myConnection, (int)index);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue removeTitlesFromPlaylist(long indices[]) {
		for ( int i = 0; i < indices.length; i++)
			if (removeTitleFromPlaylist(indices[i]-i).getType() == ReturnValue.ERROR_TYPE)
				break;
		return ret;
	}
	
	public ReturnValue setSetting(String setting, String value) {
		result = Xmmsclient.xmmsc_configval_set(myConnection, setting, value);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue creaseVolume(int val) {
		logger.debug("crease volume");
		return setVolume(mm.getVolume() + val);
	}
	
	public ReturnValue setVolume(int val) {
		logger.debug("Set volume to " + val);
		result = Xmmsclient.xmmsc_playback_volume_set(myConnection, "left", val);
		Xmmsclient.xmmsc_result_wait(result);
		if (Xmmsclient.xmmsc_result_iserror(result) == 1){
			ret.setType(ReturnValue.ERROR_TYPE);
			ret.setValue(Xmmsclient.xmmsc_result_get_error(result));
			return ret;
		}
		if (result != null) Xmmsclient.xmmsc_result_unref(result);
		result = Xmmsclient.xmmsc_playback_volume_set(myConnection, "right", val);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue setMixer(int mixer, int val) {
		if (!mm.getConfigVal("effect.equalizer.enabled").equals("1")){
			ReturnValue rt = setSetting("effect.equalizer.enabled", ""+1);
			if (rt.getType() == ReturnValue.ERROR_TYPE)
				return rt;
		}
		return setSetting("effect.equalizer.gain" + mixer, ""+(val/2.5));
	}
	
	public String spinDown(){
		String error = null;
		if (result == null)
			error = "calling the requested function didn't succeed";
		else if (Xmmsclient.xmmsc_result_iserror(result) == 1)
			error = Xmmsclient.xmmsc_result_get_error(result);
		if (result != null) Xmmsclient.xmmsc_result_unref(result);
		synchronized(Xmms2ThreadFactory.lockControlConnection){
			Xmms2ThreadFactory.lockControlConnection.notify();
			Xmms2ThreadFactory.lockControlConnection = Boolean.FALSE;
		}
		return error;
	}
	
	protected boolean enterCritical(){
		synchronized(Xmms2ThreadFactory.lockControlConnection){
			while (Xmms2ThreadFactory.lockControlConnection.equals(Boolean.TRUE)) try {
				Xmms2ThreadFactory.lockControlConnection.wait();
			} catch (InterruptedException e) {
				logger.debug(e.getMessage(), e);
				return false;
			}
			Xmms2ThreadFactory.lockControlConnection = Boolean.TRUE;
			return true;
		}
	}
}
