/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package core.xmms2;

import java.util.Observer;

import org.xmms2.Callbacks;
import org.xmms2.MainloopJava;
import org.xmms2.xmms2bindings.*;

import core.KeepStateThread;
import core.Management;
import core.Playlist;
import core.Title;
import core.UpdateUIType;

/**
 * The Xmms2KeepStateThread handles the callbacks in java (implements org.xmms2.Callbacks). It is the xmms2-specific
 * version of our KeepStateThread.The KeepStateThread uses the java-Mainloop from org.xmms2.MainloopJava.
 * For an idea what the functions do look at core.KeepStateThread and org.xmms2.Callbacks, only some things
 * are explained here
 */

final class Xmms2KeepStateThread extends KeepStateThread implements Callbacks{
	private SWIGTYPE_p_xmmsc_connection_St myConnection;
	private MainloopJava mlj;
	private Xmms2ThreadFactory tf;
	
	/**
	 * The constructor sets the connection and calls the superconstructor. Then it starts itself as a thread
	 * 
	 * @param tf			the Xmms2ThreadFactory (for reconnecting)
	 * @param ob			thread's Observer which will be informed if a config-value changes
	 * @param mm			Management
	 * @param myConnection  connection to the xmms2 server
	 */
	protected Xmms2KeepStateThread(Xmms2ThreadFactory tf, Observer ob, Management mm,
			SWIGTYPE_p_xmmsc_connection_St myConnection){
		super(ob, mm);
		this.tf = tf;
		this.myConnection = myConnection;
		new Thread(this).start();
	}
	
	public void run(){
		init();
		while (true)
			try {Thread.sleep(30);} catch (InterruptedException e) {}
	}

	/**
	 * initiate needed startup data
	 *
	 */
	private void init() {
		while (!tf.isConnected())try {Thread.sleep(300);} catch (InterruptedException e) {}
		updateAll();
		initMainloop();
	}

	/**
	 * Init Mainloop means to set all callbacks etc
	 *
	 */
	private void initMainloop() {
		mlj = new MainloopJava(this, myConnection);
		mlj.setConfigvalChangedCallback(0);
		mlj.setMedialibEntryChangedCallback(0);
		mlj.setMedialibPlaylistLoadedCallback(0);
		mlj.setPlaybackCurrentIDCallback(0);
		mlj.setPlaybackStatusCallback(0);
		mlj.setPlaylistChangedCallback(0);
		mlj.setPlaylistCurrentPositionCallback(0);
		mlj.setPlaybackPlaytimeSignal(0);
		//mlj.setVisualizationDataSignal(0);
		// TODO only useful when adding a complete path
		//mlj.setMediareaderUnindexedSignal(0);
		mlj.setPlaybackVolumeChangedCallback(0);
		mlj.setMediainfoReaderStatusCallback(0);
		mlj.start();
	}
	
	public void restart(){
		init();
	}
	
	/**
	 * updateAll - update startup data using data from clientlib
	 * 
	 */
	private void updateAll(){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.xmmsc_playback_status(myConnection);
		Xmmsclient.xmmsc_result_notifier_set(result, 
				XmmsclientConstants.CALLBACK_PLAYBACK_STATUS, new int[]{0});
		Xmmsclient.xmmsc_result_unref(result);
		result = Xmmsclient.xmmsc_playback_volume_get(myConnection);
		Xmmsclient.xmmsc_result_notifier_set(result, 
				XmmsclientConstants.CALLBACK_PLAYBACK_VOLUME_CHANGED, new int[]{0});
		Xmmsclient.xmmsc_result_unref(result);
		result = Xmmsclient.xmmsc_playlist_list(myConnection);
		Xmmsclient.xmmsc_result_notifier_set(result, 
				XmmsclientConstants.USER_DEFINED_CALLBACK_3, new int[]{0});
		Xmmsclient.xmmsc_result_unref(result);
		result = Xmmsclient.xmmsc_configval_list(myConnection);
		Xmmsclient.xmmsc_result_notifier_set(result, 
				XmmsclientConstants.CALLBACK_CONFIGVAL_CHANGED, new int[]{0});
		Xmmsclient.xmmsc_result_unref(result);
	}
	
	protected void spinDown(){
		if (mlj != null)
			mlj.spinDown();
	}
	
	protected void playlistBusy(boolean flag){}
	protected void mlibBusy(boolean flag){}
	
	protected int getEQ(int x){
		String val = "" + configs.get("effect.equalizer.gain" + x);
		if (!val.equals("") && !val.equals("null"))
			try {
				return (int)(Double.parseDouble(val)*2.5);
			} catch (Exception e){
				logger.debug(e.getMessage(), e);
			}
		return -51;
	}
	
	protected String[] getFileTypes() {
		String bleh = ""+getConfigs().get("playlist.html.suffixes");
		return bleh.split(",");
	}
	
	protected String[] getPlaylistTypes() {
		return new String[]{"pls", "m3u"};
	}
	
	protected String[] getAllSupportedTypes(){
		String bleh = ""+getConfigs().get("playlist.html.suffixes");
		bleh += ",pls,m3u";
		return bleh.split(",");
	}
	
	public void callbackConfigvalChanged(long res, int user_data){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		
		logger.debug("Updating one setting");
		Xmmsclient.xmmsc_result_dict_foreach(result, XmmsclientConstants.CALLBACK_DICT_FOREACH_FUNCTION, new int[]{1});
	}
	
	public void callbackPlaybackStatus(long res, int user_data){
		logger.debug("playbackstatus changed");
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		int state[] = new int[1];
		Xmmsclient.xmmsc_result_get_uint(result, state);
		if (state[0] == 0) setPlaybackStatus(IDLE);
		else if (state[0] == 1) setPlaybackStatus(PLAYING);
		else if (state[0] == 2) setPlaybackStatus(PAUSED);
	}
	
	private  boolean isError(SWIGTYPE_p_xmmsc_result_St result) {
		if (Xmmsclient.xmmsc_result_iserror(result) == 1){
			logger.debug("got error");
			logger.debug(Xmmsclient.xmmsc_result_get_error(result));
			return true;
		}
		return false;
	}
	
	public void callbackPlaybackID(long res, int user_data){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
	}
	
	public void callbackPlaybackVolumeChanged(long res, int user_data){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		int right[] = new int[1];
		int left[] = new int[1];
		right[0] = 0;
		left[0] = 0;
		Xmmsclient.xmmsc_result_get_dict_entry_uint32(result, "right", right);
		Xmmsclient.xmmsc_result_get_dict_entry_uint32(result, "left", left);
		setVolume((right[0]+left[0])/2);
	}
	
	public void callbackPlaylistChanged(long res, int user_data){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		SWIGTYPE_p_xmmsc_result_St result2 = Xmmsclient.xmmsc_playlist_list(myConnection);
		Xmmsclient.xmmsc_result_notifier_set(result2, XmmsclientConstants.USER_DEFINED_CALLBACK_3, new int[]{0});
		Xmmsclient.xmmsc_result_unref(result2);
	}
	
	public void callbackMedialibEntryChanged(long res, int user_data){
		logger.debug("medialibentry changed");
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		int id[] = new int[1];
		Xmmsclient.xmmsc_result_get_uint(result, id);
		logger.debug("medialibentry no " + id[0] + " changed");
		mm.updateTitleCache((int)id[0]);
		setChanged();
		notifyObservers(new UpdateUIType(UpdateUIType.MLIB_ENTRY_CHANGED, new Integer((int)id[0])));
	}
	
	public  void callbackMedialibPlaylistLoaded(long res, int user_data){
		logger.debug("loaded playlist");
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		setChanged();
		notifyObservers(new UpdateUIType(UpdateUIType.PLAYLIST_LOADED, null));
	}
	
	public  void callbackPlaylistCurrentPosition(long res, int user_data){
		logger.debug("playlistposition changed");
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		
		int[] pos = new int[1];
		Xmmsclient.xmmsc_result_get_uint(result, pos);
		setCurrentPlaylistIndex(pos[0]);
	}
	
	public  void callbackDisconnect(int error){
		tf.connect();
	}
	
	public void signalPlaybackPlaytime(long res, int user_data){
		final SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		int time[] = new int[1];
		Xmmsclient.xmmsc_result_get_uint(result, time);
		updatePlaytime(time[0]/1000.0);
		/*new Timer().schedule(new TimerTask(){
			public void run() {
				SWIGTYPE_p_xmmsc_result_St res2 = Xmmsclient.xmmsc_result_restart(result);
				if (isError(res2))
					return;
				Xmmsclient.xmmsc_result_unref(result);
				Xmmsclient.xmmsc_result_unref(res2);
			}
		}, 100);*/
		SWIGTYPE_p_xmmsc_result_St res2 = Xmmsclient.xmmsc_result_restart(result);
		Xmmsclient.xmmsc_result_unref(result);
		Xmmsclient.xmmsc_result_unref(res2);
	}
	
	public void signalVisualizationData(long res, int user_data){
		/*final SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		
		new Timer().schedule(new TimerTask(){
			public void run() {
				SWIGTYPE_p_xmmsc_result_St res2 = Xmmsclient.xmmsc_result_restart(result);
				if (isError(res2))
					return;
				Xmmsclient.xmmsc_result_unref(result);
				Xmmsclient.xmmsc_result_unref(res2);
			}
		}, 150);
		*/
	}
	
	/**
	 * Callback is used to fill configlist on startup
	 */
	public void callbackDictForeachFunction(String key, int type, String value, int user_data) {
		if (user_data == 1){
			if (key != null && !key.equals("")){
				if (!configs.containsKey(key))
					configs.put(key, value);
				else updateConfig(key, value);
				if (key.startsWith("effect.equalizer.gain")){
					setChanged();
					notifyObservers(new UpdateUIType(UpdateUIType.VOLUME_OR_EQUALIZER_CHANGED, 
							new Integer(key.replaceAll("effect.equalizer.gain", ""))));
				}
			}
		}
		else System.out.println("dict: " + key + "\t" + value);
	}
	
	/**
	 * handle arrival of new playlist
	 */
	public void userDefinedCallback3(long arg0, int arg1) {
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(arg0);
		if (Xmmsclient.xmmsc_result_is_list(result) == 1){
			Playlist pl = new Playlist();
			Xmmsclient.xmmsc_result_list_first(result);
			int id[] = new int[1];
			while (Xmmsclient.xmmsc_result_list_valid(result) == 1){
				Xmmsclient.xmmsc_result_get_uint(result, id);
				Title t = new Title();
				t.setAttribute("id",""+id[0]);
				pl.add(t);
				Xmmsclient.xmmsc_result_list_next(result);
			}
			setCurrentPlaylist(pl);
			if (pl.getSize() > 0){
				SWIGTYPE_p_xmmsc_result_St result2 = Xmmsclient.xmmsc_playlist_current_pos(myConnection);
				Xmmsclient.xmmsc_result_notifier_set(result2, XmmsclientConstants.CALLBACK_PLAYLIST_CURRENT_POSITION, new int[]{0});
				Xmmsclient.xmmsc_result_unref(result2);
			}
			else setCurrentPlaylistIndex(0);
		}
	}
	
	public void callbackMediainfoReaderStatus(long arg0, int arg1) {
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(arg0);
		if (isError(result))
			return;
		int state[] = new int[1];
		Xmmsclient.xmmsc_result_get_int(result, state);
		setChanged();
		if (state[0] == 1)
			setBackgroundWorkerStatus(UpdateUIType.BG_WORKING);
		else setBackgroundWorkerStatus(UpdateUIType.BG_IDLE);
		notifyObservers(new UpdateUIType(UpdateUIType.BG_WORKING, null));
	}
	
	public void signalMediareaderUnindexed(long res, int arg1) {
		final SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		int todo[] = new int[1];
		Xmmsclient.xmmsc_result_get_uint(result, todo);
		
		setBackgroundWorkerStatus(UpdateUIType.BG_WORKING+((int)todo[0]));
		
		SWIGTYPE_p_xmmsc_result_St res2 = Xmmsclient.xmmsc_result_restart(result);
		Xmmsclient.xmmsc_result_unref(result);
		Xmmsclient.xmmsc_result_unref(res2);
	}
	
	public void lockFunction(int v) {}
	public void unlockFunction(int v) {}
	public void userDefinedCallback1(long arg0, int arg1) {}
	public void callbackMedialibEntryAdded(long arg0, int arg1) {}
	public void callbackPropdictForeachFunction(String key, int type, String value, String source, int user_data) {}
	public void userDefinedCallback2(long arg0, int arg1) {}
}
