/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package gui;

import java.util.ArrayList;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XLabel;
import org.Xwidgets.XMenu;
import org.Xwidgets.XPanel;
import org.Xwidgets.XSlider;
import org.apache.log4j.Logger;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.jdom.Element;

import core.Management;

import statics.StaticValues;

/**
 * EQPanel represents the UI-Part of the Equalizer. It provides a XPanel with n Equalizer sliders and labels.
 * It is, as every other ui-class singleton.
 */

public class EQPanel {
	private static EQPanel instance = null;
	protected XPanel panel;
	protected XMenu menu;
	protected Object layout, layoutData, separate, rowData;
	private Logger logger;
	private WidgetFactory wt;
	protected ArrayList labels, sliders;
	protected int eqCount = 0;
	
	/**
	 * Constructor for EQPanel - usually called only once cause it's singleton
	 * 
	 * @param parent
	 * @param panel
	 * @param eqCount
	 */
	private EQPanel(XPanel parent, Element panel, int eqCount){
		try {
			logger = Logger.getLogger(getClass());
			wt = MainGUI.mainGui.myGUIFactory;
			this.eqCount = eqCount;
			setLayout();
			buildMe(parent, panel);
		}catch (Exception e){
			logger.debug(e.getMessage(), e);
		}
	}
	
	/**
	 * Set layoutdata for panels
	 *
	 */
	private void setLayout() {
		if (StaticValues.getSetting("gui").equals("swt")){
			layout = new GridLayout(eqCount, false);
			((GridLayout)layout).horizontalSpacing = 0;
			
			layoutData = new GridData(GridData.FILL_BOTH);
			((GridData)layoutData).grabExcessVerticalSpace = true;
			((GridData)layoutData).grabExcessHorizontalSpace = true;
			((GridData)layoutData).widthHint = 40;
			
			separate = new RowLayout(Styles.VERTICAL);
			((RowLayout)separate).fill = true;
			((RowLayout)separate).justify = true;
			
			rowData = new RowData();
			((RowData)rowData).width = 35;
		}
	}

	/**
	 * Create or return the existing EQPanel
	 * 
	 * @param parent		the EQPanel's parent
	 * @param panel		jdom Element describing the EQPanel
	 * @param eqCount		No of Equalizers (usually got from the Management)
	 * @return				a EQPanel object
	 */
	
	protected static EQPanel getEQPanel(XPanel parent, Element panel, int eqCount){
		if (instance == null)
			instance = new EQPanel(parent, panel, eqCount);
		return instance;
	}

	/**
	 * Called by Constructor to initialize the thing
	 * 
	 * @param parent
	 * @param p
	 */
	private void buildMe(XPanel parent, Element p) {
		try {
			sliders = new ArrayList();
			labels = new ArrayList();
			
			panel = wt.getPanel(parent, -1, -1, Styles.SIMPLE);
			panel.setLayout(layout);
			panel.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			
			for (int i = 0; i < eqCount; i++){
				XPanel tmp = wt.getPanel(panel, -1, -1, Styles.BORDER);
				tmp.setLayout(separate);
				tmp.setLayoutData(layoutData);
				XSlider s = wt.getSlider(tmp, -1, -1, Styles.VERTICAL);
				s.addMouseListener(MouseListener.getInstance());
				s.setMax(100);
				s.setPosition((Management.getInstance().getEQ(i)-50)*(-1));
				s.setLayoutData(rowData);
				sliders.add(s);
				XLabel l = wt.getLabel(tmp, -1, -1, ""+Management.getInstance().getEQ(i), Styles.CENTER);
				l.setLayoutData(rowData);
				labels.add(l);
			}
			
			menu = wt.getDropDownMenu(panel);
			panel.setMenu(menu);
		} catch (Exception e) {
			logger.debug(e.getMessage(), e);
		}
	}
}
