/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package gui;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XButton;
import org.Xwidgets.XCheckBox;
import org.Xwidgets.XComboBox;
import org.Xwidgets.XLabel;
import org.Xwidgets.XMenu;
import org.Xwidgets.XMenuItem;
import org.Xwidgets.XPanel;
import org.Xwidgets.XTable;
import org.Xwidgets.XTextField;
import org.Xwidgets.XWidgetsException;
import org.apache.log4j.Logger;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.jdom.Element;

import statics.StaticValues;

/**
 * Class represents the Medialib part of the UI. It provides to panels, search- and browsePanel which can be used elsewhere.
 * 
 */
class MlibPanel {
	protected XTable searchTable, browseTable;
	protected XTextField textSearch;
	protected XPanel searchPanel, browsePanel;
	protected XComboBox searchBox, browseBox;
	protected XButton searchButton;
	protected XCheckBox strict;
	protected XMenu menuMedial, browseMenu;
	protected XMenuItem addToPlaylist, addEntryMedialib, addRandomEntries, addDirectory,
		searchFor, browseNetwork, titleInfo, refreshMlib;
	protected XLabel preSearchbox;
	private XPanel parent;
	private Element mlib;
	private WidgetFactory wt;
	private Logger logger;
	private static MlibPanel instance;
	protected Object layout, layoutData, rows, rowsLayout;
	
	private MlibPanel(XPanel parent, Element mlib, Object panelLayout, Object layoutData) {
		logger = Logger.getLogger(getClass());
		this.parent = parent;
		this.mlib = mlib;
		layout = panelLayout;
		this.layoutData = layoutData;
		wt = MainGUI.mainGui.myGUIFactory;
		buildMe();
	}
	
	/**
	 * Get and/or build the panels
	 * 
	 * @param parent			parent element of the panels, maybe a dummy
	 * @param mlib				Element containing the builddata for both panels
	 * @param panelLayout		layoutdata (with/without borders)
	 * @param layoutData		how to fill the layout
	 * @return					return a MlibPanel
	 */
	public static MlibPanel getInstance(XPanel parent, Element mlib, Object panelLayout, Object layoutData){
		if (instance == null)
			instance = new MlibPanel(parent, mlib, panelLayout, layoutData);
		return instance;
	}
	
	/**
	 * Set Layout(data) for later use
	 *
	 */
	private void setLayout(){
		if (StaticValues.getSetting("gui").equals("swt")){
			rows = new GridLayout(3, false);
			rowsLayout = new GridData(GridData.FILL_HORIZONTAL);
			((GridData)rowsLayout).grabExcessHorizontalSpace = true;
			((GridData)rowsLayout).heightHint = 35;
		}
	}
	
	/**
	 * Called by Constructor to build Elements
	 *
	 */
	private void buildMe(){
		try {
			setLayout();
			buildSearchPanel();
			buildBrowsePanel();
		} catch (XWidgetsException e) {
			logger.debug(e.getMessage(), e);
		}
	}
	
	/**
	 * used to get a "key" from a given Combobox. Useful in some Listeners for building Mlib queries
	 * 
	 * @param box		The XComboBox where you want to get your key from
	 * @return			the key, "" otherwise
	 */
	protected static String getTextFromComboIndex(XComboBox box){
		int index = box.getSelectedIndex();
		if (box.getRealObject().equals(MainGUI.mainGui.mlibPanel.searchBox.getRealObject())){
			switch (index){
				case 0:		return "artist";
				case 1:		return "title";
				case 2:		return "album";
				case 3:		return "all";
				case 4:		return "ownQuery";
				default:	return "";
			}
		}
		else if (box.getRealObject().equals(MainGUI.mainGui.mlibPanel.browseBox.getRealObject())){
			switch (index){
				case 0:		return "artist";
				case 1:		return "album";
				case 2:		return "genre";
				default:	return "";
			}
		}
		return "";
	}
	
	/**
	 * Constructs the Searchpanel
	 * @throws XWidgetsException		if layout is not ok
	 */
	private void buildSearchPanel() throws XWidgetsException {
		searchPanel = wt.getPanel(parent, -1, -1, Styles.BORDER);
		searchPanel.setLayout(layout);
		
		XLabel header = wt.getLabel(searchPanel, -1, -1, StaticValues.getLangString("mlibSearch").toUpperCase(), 
				Styles.CENTER);
		header.setLayoutData(layoutData);
		header.setTooltipText(StaticValues.getLangString("mlibSearchHeader"));
		
		searchBox = wt.getComboBox(searchPanel,-1, -1,Styles.BORDER|Styles.READ_ONLY, 
				new String[]{StaticValues.getLangString("artist"),StaticValues.getLangString("title"),
				StaticValues.getLangString("album"),StaticValues.getLangString("allTitles"), 
				StaticValues.getLangString("ownQuery")});
		searchBox.setLayoutData(layoutData);
		searchBox.addSelectionListener(SelectionListener.getInstance());
		
		XPanel tmp = wt.getPanel(searchPanel, -1, -1, Styles.BORDER);
		tmp.setLayout(rows);
		tmp.setLayoutData(rowsLayout);
		
		preSearchbox = wt.getLabel(tmp, -1, -1, StaticValues.getLangString("searchFor"), Styles.LEFT);
		
		textSearch = wt.getTextField(tmp, -1, -1, "", Styles.BORDER|Styles.SINGLE);
		textSearch.setLayoutData(layoutData);
		textSearch.addKeyListener(KeyListener.getInstance());
		
		strict = wt.getCheckBox(tmp, -1, -1, StaticValues.getLangString("strict"), Styles.NULL);
		strict.setTooltipText(StaticValues.getLangString("exactMeans"));
		
		searchButton = wt.getButton(searchPanel, -1, -1,StaticValues.getLangString("search"), Styles.CENTER);
		searchButton.setLayoutData(layoutData);
		searchButton.addMouseListener(MouseListener.getInstance());
		
		searchTable = wt.getTable(searchPanel, -1, -1,
			Styles.BORDER|Styles.MULTI, new String[]{StaticValues.getLangString("id"),
			StaticValues.getLangString("artist"),StaticValues.getLangString("title"),
			StaticValues.getLangString("album"),StaticValues.getLangString("genre")}, 
			new int[]{45,135,250,135,135});
		searchTable.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
		searchTable.addMouseListener(MouseListener.getInstance());
		
		menuMedial = wt.getDropDownMenu(searchTable);
		menuMedial.addMenuListener(MenuListener.getInstance());
		addToPlaylist = wt.getMenuItem(menuMedial);
		addToPlaylist.setText(StaticValues.getLangString("addToPlaylist"));
		addToPlaylist.addSelectionListener(SelectionListener.getInstance());
		titleInfo = wt.getMenuItem(menuMedial);
		titleInfo.setText(StaticValues.getLangString("showPlaylistEntry"));
		titleInfo.addSelectionListener(SelectionListener.getInstance());
		searchTable.setMenu(menuMedial);
	}
	
	/**
	 * Constructs Browsepanel
	 * 
	 * @throws XWidgetsException		if layout is not ok
	 */
	private void buildBrowsePanel() throws XWidgetsException {
		browsePanel = wt.getPanel(parent, 0, 0, Styles.BORDER);
		browsePanel.setLayout(layout);
		
		XLabel header = wt.getLabel(browsePanel, -1, -1, StaticValues.getLangString("mlibBrowse").toUpperCase(), 
				Styles.CENTER);
		header.setLayoutData(layoutData);
		header.setTooltipText(StaticValues.getLangString("mlibBrowseHeader"));
		
		browseBox = wt.getComboBox(browsePanel,-1, -1,Styles.BORDER|Styles.READ_ONLY,
				new String[]{StaticValues.getLangString("artist"),
				StaticValues.getLangString("album"),StaticValues.getLangString("genre")});
		browseBox.setLayoutData(layoutData);
		browseBox.addSelectionListener(SelectionListener.getInstance());
		
		browseTable = wt.getTable(browsePanel, -1, -1, Styles.BORDER|Styles.MULTI, new String[]{
				StaticValues.getLangString("contentBrowse")}, new int[]{300});
		browseTable.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
		browseTable.addMouseListener(MouseListener.getInstance());
		
		browseMenu = wt.getDropDownMenu(browsePanel);
		addEntryMedialib = wt.getMenuItem(browseMenu);
		addEntryMedialib.setText(StaticValues.getLangString("mlibAdd"));
		addEntryMedialib.addSelectionListener(SelectionListener.getInstance());
		addDirectory = wt.getMenuItem(browseMenu);
		addDirectory.setText(StaticValues.getLangString("mlibAddDir"));
		addDirectory.addSelectionListener(SelectionListener.getInstance());
		browseNetwork = wt.getMenuItem(browseMenu);
		browseNetwork.setText(StaticValues.getLangString("mlibBrowseNetwork"));
		browseNetwork.addSelectionListener(SelectionListener.getInstance());
		searchFor = wt.getMenuItem(browseMenu);
		searchFor.setText(StaticValues.getLangString("searchFor"));
		searchFor.addSelectionListener(SelectionListener.getInstance());
		addRandomEntries = wt.getMenuItem(browseMenu);
		addRandomEntries.setText(StaticValues.getLangString("mlibAddRandom"));
		addRandomEntries.addSelectionListener(SelectionListener.getInstance());
		refreshMlib = wt.getMenuItem(browseMenu);
		refreshMlib.setText(StaticValues.getLangString("refreshMlib"));
		refreshMlib.addSelectionListener(SelectionListener.getInstance());
		browseMenu.addMenuListener(MenuListener.getInstance());
		browseTable.setMenu(browseMenu);
	}
}

