/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * Copyright Georg Schild (dangertools@gmail.com), Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package gui;

import java.io.File;
import java.io.FilenameFilter;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.ListIterator;
import java.util.regex.Pattern;

import jcifs.smb.SmbException;
import jcifs.smb.SmbFile;

import org.Xwidgets.Styles;
import org.Xwidgets.XButton;
import org.Xwidgets.XComboBox;
import org.Xwidgets.XDirectoryChooser;
import org.Xwidgets.XRectangle;
import org.Xwidgets.XSlider;
import org.Xwidgets.XTableItem;
import org.Xwidgets.XTextField;
import org.Xwidgets.event.XFocusAdapter;
import org.Xwidgets.event.XFocusEvent;
import org.Xwidgets.event.XKeyAdapter;
import org.Xwidgets.event.XKeyEvent;
import org.Xwidgets.event.XMouseAdapter;
import org.Xwidgets.event.XMouseEvent;
import org.Xwidgets.event.XSelectionAdapter;
import org.Xwidgets.event.XSelectionEvent;
import org.apache.log4j.Logger;

import statics.StaticValues;

import core.Management;
import core.Title;

/**
 * Packagemouselistener which handles all "clicks"
 */

class MouseListener extends XMouseAdapter {
	private Management myManagement;
	private Logger logger;
	private static MouseListener instance;
	
	private MouseListener(){
		logger = Logger.getLogger(getClass());
		myManagement = Management.getInstance();
	}

	/**
	 * Get a Listener. Call this method to set a listener on a control
	 * 
	 * @return		a MouseListener
	 */
	public static MouseListener getInstance(){
		if (instance == null)
			instance = new MouseListener();
		return instance;
	}
	
	/**
	 * Handle a MouseDoubleClick event
	 */
	public void xMouseDoubleClick(XMouseEvent e){
		if (e.getSource().equals(MainGUI.mainGui.prefGui.backendPrefs.getRealObject())){
			final XTableItem xti = MainGUI.mainGui.prefGui.backendPrefs.getItemForPoint(e.x, e.y);
			if (xti != null){
				int index = xti.getIndex(e.x, e.y);
				if (index == 1){
					XRectangle rect = xti.getXBounds(index);
					final XTextField xtf = MainGUI.mainGui.myGUIFactory.getTextField(MainGUI.mainGui.prefGui.backendPrefs, rect.getWidth(),
							rect.getHeigth(), xti.getText(index), Styles.SINGLE);
					xtf.setLocation((int)rect.getX(), (int)rect.getY());
					xtf.forceFocus();
					xtf.addFocusListener(new XFocusAdapter(){
						public void xFocusLost(XFocusEvent arg0) {
							xtf.dispose();
						}
						public void xFocusGained(XFocusEvent event) {}
					});
					xtf.addKeyListener(new XKeyAdapter(){
						public void xKeyPressed(XKeyEvent arg0) {
							if (arg0.character == (char)13){
								myManagement.setConfigVal(xti.getText(0), xtf.getText());
								xti.setText(1, xtf.getText());
								xtf.dispose();
							}
						}
						public void xKeyReleased(XKeyEvent event) {}
					});
				}
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.prefGui.frontendPrefs.getRealObject())){
			final XTableItem xti = MainGUI.mainGui.prefGui.frontendPrefs.getItemForPoint(e.x, e.y);
			if (xti != null){
				int index = xti.getIndex(e.x, e.y);
				if (index == 1){
					XRectangle rect = xti.getXBounds(index);
					final String key = xti.getText(0);
					final String value = xti.getText(1);
					if (key.toLowerCase().endsWith("dir")){
						final XButton chooser = MainGUI.mainGui.myGUIFactory.getButton(MainGUI.mainGui.prefGui.frontendPrefs,
								rect.getHeigth(), rect.getHeigth(), "...", Styles.SIMPLE);
						chooser.setLocation((int)rect.getX(), (int)rect.getY());
						chooser.forceFocus();
						chooser.addFocusListener(new XFocusAdapter(){
							public void xFocusGained(XFocusEvent event) {}
							public void xFocusLost(XFocusEvent event) {
								chooser.dispose();
							}
						});
						chooser.addMouseListener(new XMouseAdapter(){
							public void xMouseDown(XMouseEvent arg0) {
								final XDirectoryChooser xdc = MainGUI.mainGui.myGUIFactory.
									getDirectoryChooser(MainGUI.mainGui.prefGui.myFrame);
								xdc.setSelectedPath(value);
								xdc.open();
								String newPath = xdc.getSelectedPath();
								if (newPath != null && !newPath.equals("")){
									StaticValues.setSetting(key, newPath);
									xti.setText(1, newPath);
									chooser.dispose();
								}
							}
							public void xMouseDoubleClick(XMouseEvent arg0) {}
							public void xMouseUp(XMouseEvent arg0) {}
						});
					}
					else if (key.toLowerCase().startsWith("server")){
						final XTextField xtf = MainGUI.mainGui.myGUIFactory.getTextField(MainGUI.mainGui.prefGui.frontendPrefs,
								rect.getWidth(), rect.getHeigth(), value, Styles.SINGLE);
						xtf.setLocation((int)rect.getX(), (int)rect.getY());
						xtf.forceFocus();
						xtf.addFocusListener(new XFocusAdapter(){
							public void xFocusGained(XFocusEvent event) {}
							public void xFocusLost(XFocusEvent event) {
								xtf.dispose();
							}
						});
						xtf.addKeyListener(new XKeyAdapter(){
							public void xKeyPressed(XKeyEvent event) {
								if (event.character == 27)
									xtf.dispose();
								else if (event.character == 13){
									String input = xtf.getText();
									boolean accept = true;
									if (input.length() == 0)
										accept = true;
									else if (key.equals("serverport")){
										try {
											int port = Integer.parseInt(input);
											if (port < 0 || port > 65535)
												accept = false;
										} catch (Exception e1){
											accept = false;
										}
									}
									else {
										final String TRIPLE = "(((1)?[0-9]{1,2})|(2(([0-4][0-9])|(5[0-5]))))";
										final String IP = "("+TRIPLE+"\\.){3}" + TRIPLE;
										accept = Pattern.matches(IP, input);
									}
									if (accept){
										StaticValues.setSetting(key, input);
										xti.setText(1, ""+input);
									}
									xtf.dispose();
								}
							}

							public void xKeyReleased(XKeyEvent event) {}
						});
					}
					else {
						try {
							String entries[] = null;
							if (key.equalsIgnoreCase("iconStyle") || key.equalsIgnoreCase("style")){
								File f = null;
								if (key.equals("iconstyle"))
									f = new File(StaticValues.getSetting("icondir"));
								else
									f = new File(StaticValues.getSetting("styledir"));
								entries = f.list(new FilenameFilter(){
									public boolean accept(File arg0, String arg1) {
										File f1 = new File(arg0, arg1);
										if (key.equals("style") && f1.isFile()){
											if (arg1.endsWith(".xml")) return true;
											return false;
										}
										if (f1.isDirectory() && !arg1.equalsIgnoreCase("cvs"))
											return true;
										return false;
									}
								});
							}
							else if (key.equalsIgnoreCase("backend"))
								entries = new String[]{"xmms2"};
							else if (key.equalsIgnoreCase("gui"))
								entries = new String[]{"swt"};
							else {
								logger.debug("We should have never reached that point!"); return;
							}
							Arrays.sort(entries);
							if (entries.length > 1){
								for ( int i = 0; i < entries.length; i++)
									entries[i] = entries[i].replaceAll("\\.xml", "");
								final XComboBox xcb = MainGUI.mainGui.myGUIFactory.getComboBox(MainGUI.mainGui.prefGui.frontendPrefs,
										rect.getWidth(), rect.getHeigth(), Styles.SINGLE|Styles.READ_ONLY, entries);
								xcb.setLocation((int)rect.getX(), (int)rect.getY());
								xcb.forceFocus();
								xcb.setSelectedIndex(xcb.indexOf(value));
								xcb.addSelectionListener(new XSelectionAdapter(){
									public void xWidgetSelected(XSelectionEvent arg0) {
										StaticValues.setSetting(key, ""+xcb.getItemAt(xcb.getSelectedIndex()));
										xti.setText(1, ""+xcb.getItemAt(xcb.getSelectedIndex()));
										xcb.dispose();
									}
									public void xWidgetDefaultSelected(XSelectionEvent arg0) {}
								});
								xcb.addKeyListener(new XKeyAdapter(){
									public void xKeyPressed(XKeyEvent event) {
										xcb.dispose();
									}
									public void xKeyReleased(XKeyEvent event) {}
								});
							}
						} catch (Exception e1){
							logger.debug(e1.getMessage(), e1);
						}
					}
				}
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.table.getRealObject())){
			int items[] = MainGUI.mainGui.playlistPanel.table.getSelectedIndices();
			if (items.length > 0){
				myManagement.nextInPlaylistAbs(items[0]);
				myManagement.play();
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.mlibPanel.browseTable.getRealObject())) {
			int items[] = MainGUI.mainGui.mlibPanel.browseTable.getSelectedIndices();
			if (items.length > 0){
				String[] query = new String[3];
				query[0] = MlibPanel.getTextFromComboIndex(MainGUI.mainGui.mlibPanel.browseBox);
				query[1] = MainGUI.mainGui.mlibPanel.browseTable.getItemForIndex(items[0]).getText(0);
				query[2] = "exact";
				myManagement.mlibSelect(query, true);
				MainGUI.mainGui.mlibPanel.textSearch.setText(query[1]);
				MainGUI.mainGui.mlibPanel.strict.setChecked(true);
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.mlibPanel.searchTable.getRealObject())){
			int items[] = MainGUI.mainGui.mlibPanel.searchTable.getSelectedIndices();
			if (items.length > 0){
				for ( int i = 0; i < items.length; i++){
					myManagement.addIdToPlaylist(new Integer(MainGUI.mainGui.mlibPanel.searchTable.getItemForIndex
							(items[i]).getText(0)).intValue());
					logger.debug("id: " + new Integer(MainGUI.mainGui.mlibPanel.searchTable.getItemForIndex
							(items[i]).getText(0)).intValue());
				}
				if (MainGUI.mainGui.tabbedPane != null)
					MainGUI.mainGui.tabbedPane.setSelection(MainGUI.mainGui.
							tabbedPane.getIndex(MainGUI.mainGui.playlistPanel.panel));
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.playlistTable.getRealObject())){
			int items[] = MainGUI.mainGui.playlistPanel.playlistTable.getSelectedIndices();
			if (items.length > 0){
				MainGUI.mainGui.playlistPanel.table.clear();
				myManagement.mlibLoadPlaylist(MainGUI.mainGui.playlistPanel.playlistTable.
						getItemForIndex(items[0]).getText(1));
				if (MainGUI.mainGui.tabbedPane != null)
					MainGUI.mainGui.tabbedPane.setSelection(MainGUI.mainGui.tabbedPane.getIndex(MainGUI.mainGui.playlistPanel.panel));
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.CIFSBrowser.table.getRealObject())){
			try {
				int items[] = MainGUI.mainGui.CIFSBrowser.table.getSelectedIndices();
				if (items.length > 0){
					SmbFile tmp = new SmbFile(MainGUI.mainGui.CIFSBrowser.current, 
							MainGUI.mainGui.CIFSBrowser.table.getItemForIndex(items[0]).getText(0) + "/");
					if (tmp.isFile())
						handleNetworkBrowser(new int[]{items[0]});
					else MainGUI.mainGui.CIFSBrowser.fillTable(tmp);
				}
			} catch (MalformedURLException e1) {
				logger.debug(e1.getMessage(), e1);
			} catch (UnknownHostException e1) {
				logger.debug(e1.getMessage(), e1);
			} catch (SmbException e1) {
				logger.debug(e1.getMessage(), e1);
			}
		}
	}
	
	/**
	 * Handle a MouseSingleClick event
	 */
	public void xMouseDown(XMouseEvent e){
		if (e.getSource().equals(MainGUI.mainGui.buttonPanel.play.getRealObject())){
			myManagement.play();
			logger.debug("got play event");
		}
		else if (e.getSource().equals(MainGUI.mainGui.CIFSBrowser.close.getRealObject()))
			MainGUI.mainGui.CIFSBrowser.close();
		else if (e.getSource().equals(MainGUI.mainGui.CIFSBrowser.select.getRealObject())){
			handleNetworkBrowser(MainGUI.mainGui.CIFSBrowser.table.getSelectedIndices());
			MainGUI.mainGui.CIFSBrowser.close();
		}
		else if (e.getSource().equals(MainGUI.mainGui.buttonPanel.stop.getRealObject())){
			myManagement.stop();
			logger.debug("got stop event");
		}		
		else if (e.getSource().equals(MainGUI.mainGui.buttonPanel.pause.getRealObject())){
			myManagement.pause();
			logger.debug("got pause event");
		}
		else if (e.getSource().equals(MainGUI.mainGui.buttonPanel.next.getRealObject())){
			myManagement.next();
			logger.debug("got next event");
		}
		else if (e.getSource().equals(MainGUI.mainGui.buttonPanel.prev.getRealObject())){
			myManagement.prev();
			logger.debug("got prev event");
		}
		else if (MainGUI.mainGui.buttonPanel.about != null && e.getSource().equals(MainGUI.mainGui.buttonPanel.about.getRealObject())){
			new AboutWindow();
		}
		else if (MainGUI.mainGui.lists != null && e.getSource().equals(MainGUI.mainGui.buttonPanel.lists.getRealObject())){
			if (MainGUI.mainGui.tabbedPane != null)
				MainGUI.mainGui.tabbedPane.setSelection(0);
			MainGUI.mainGui.lists.setVisible(true);
		}
		/* propGui */
		else if (e.getSource().equals(MainGUI.mainGui.propGui.okButton.getRealObject())) {
			PropGUI p = MainGUI.mainGui.propGui;
			Title up = closePropGUI();
			up.setAttribute("artist", p.textArtist.getText());
			up.setAttribute("album", p.textAlbum.getText());
			up.setAttribute("comment", p.textComment.getText());
			up.setAttribute("title", p.textTitle.getText());
			up.setAttribute("url", p.textURL.getText());
			myManagement.mlibUpdateTitle(up);
		}
		else if (e.getSource().equals(MainGUI.mainGui.propGui.cancelButton.getRealObject())) {
			closePropGUI();
		}
		else if (e.getSource().equals(MainGUI.mainGui.prefGui.closeButton.getRealObject())) {
			closePrefGUI();
		}
		
		/* mlib */
		else if (e.getSource().equals(MainGUI.mainGui.mlibPanel.searchButton.getRealObject())) {
			searchMlib();
		}
		else if (e.getSource().equals(MainGUI.mainGui.CIFSBrowser.oneUp.getRealObject())){
			try {
				MainGUI.mainGui.CIFSBrowser.fillTable(new SmbFile(MainGUI.mainGui.CIFSBrowser.current, "../."));
			} catch (MalformedURLException e1) {
				logger.debug(e1.getMessage(), e1);
			} catch (UnknownHostException e1) {
				logger.debug(e1.getMessage(), e1);
			}
		}
	}

	/**
	 * Handle the networkbrowser use. called when user clicks ok in the networkbrowser
	 * 
	 * @param index
	 */
	private void handleNetworkBrowser(int[] index) {
		if (index.length > 0){
			for (int i = 0; i < index.length; i++){
				try {
					SmbFile tmp = new SmbFile(MainGUI.mainGui.CIFSBrowser.current, 
							MainGUI.mainGui.CIFSBrowser.table.getItemForIndex(index[i]).getText(0) + "/");
					if (tmp.isFile()){
						if (MainGUI.mainGui.CIFSBrowser.getSource().equals(MainGUI.mainGui.mlibPanel.browseNetwork))
							myManagement.mlibAddEntry(tmp.getURL().toString());
						else if (MainGUI.mainGui.CIFSBrowser.getSource().equals(MainGUI.mainGui.playlistPanel.browseNetwork))
							myManagement.addTitleToPlaylist(tmp.getURL().toString());
					}
					else if (tmp.isDirectory()){
						List l = new ArrayList();
						if (MainGUI.mainGui.CIFSBrowser.getSource().equals(MainGUI.mainGui.mlibPanel.browseNetwork)){
							recurseDir(tmp, l);
							myManagement.addURLListToMedialib(l);
						}
						else if (MainGUI.mainGui.CIFSBrowser.getSource().equals(MainGUI.mainGui.playlistPanel.browseNetwork)){
							recurseDir(tmp, l);
							myManagement.addURLListToPlaylist(l);
						}
					}
				} catch (MalformedURLException e1) {
					logger.debug(e1.getMessage(), e1);
				} catch (UnknownHostException e1) {
					logger.debug(e1.getMessage(), e1);
				} catch (SmbException e1) {
					logger.debug(e1.getMessage(), e1);
				} catch (SecurityException e1) {
					logger.debug(e1.getMessage(), e1);
				}
			}
		}
	}
	
	/**
	 * Recurse over CIFS-Directory
	 * 
	 * @param file
	 * @param l
	 */
	private void recurseDir(SmbFile file, List l) {
		try {
			if (file.exists() && file.isDirectory()){
				SmbFile list[] = file.listFiles(MainGUI.mainGui.CIFSBrowser.filter);
				for ( int i = 0; i < list.length; i++){
					if (list[i].isDirectory())
						recurseDir(list[i], l);
					else 
						l.add(list[i].getURL().toString());
				}
			}
		} catch (IllegalArgumentException e) {
			logger.debug(e.getMessage(), e);
		} catch (SmbException e) {
			logger.debug(e.getMessage(), e);
		}
	}
	
	/**
	 * Handle a MouseSingleClockRelease event
	 */
	public void xMouseUp(XMouseEvent e){
		int tmp = -1;
		if (e.getSource().equals(MainGUI.mainGui.volume.getRealObject()))
			myManagement.setVolume(MainGUI.mainGui.volume.getPosition());
		else if (e.getSource().equals(MainGUI.mainGui.playtime.getRealObject()))
			myManagement.seek(MainGUI.mainGui.playtime.getPosition());
		else if ((tmp = isGainSlider(e.getSource())) >= 0){
			int newVal = ((XSlider)MainGUI.mainGui.eqPanel.sliders.get(tmp)).getPosition()-50;
			myManagement.setEQ(tmp, newVal*(-1));
		}
	}
	
	/**
	 * Check if object is a equalizer Element
	 * 
	 * @param i
	 * @return		index of slider or -1 if it isn't a slider
	 */
	private int isGainSlider(Object i){
		if (MainGUI.mainGui.eqPanel != null)
			for (ListIterator it = MainGUI.mainGui.eqPanel.sliders.listIterator(); it.hasNext();){
				XSlider sl = (XSlider)it.next();
				if (sl.getRealObject().equals(i))
					return it.nextIndex()-1;
			}
		return -1;
	}
	
	/**
	 * Handle search mlib events
	 *
	 */
	private void searchMlib() {
		String[] query = new String[3];
		query[0] = MlibPanel.getTextFromComboIndex(MainGUI.mainGui.mlibPanel.searchBox);
		query[1] = "";
		query[2] = "";

		query[1] = MainGUI.mainGui.mlibPanel.textSearch.getText();
		if (MainGUI.mainGui.mlibPanel.strict.isChecked())
			query[2] = "exact";
		else query[1] = "%" + query[1] + "%";
		if (query[0].equals("ownQuery")) {
			query[2]="own";
			query[0]=MainGUI.mainGui.mlibPanel.textSearch.getText();
		}
		myManagement.mlibSelect(query, true);
	}
	
	/**
	 * Close properties GUI and do things
	 * 
	 * @return		return Title propGUI holds atm
	 */
	private Title closePropGUI() {
		return MainGUI.mainGui.propGui.close();
	}
	
	/**
	 * Close preferences GUI and do things
	 *
	 */
	private void closePrefGUI() {
		MainGUI.mainGui.prefGui.close();
	}
}
