/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package gui;

import java.net.MalformedURLException;
import java.net.UnknownHostException;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XArrowButton;
import org.Xwidgets.XButton;
import org.Xwidgets.XFrame;
import org.Xwidgets.XLabel;
import org.Xwidgets.XPanel;
import org.Xwidgets.XTable;
import org.Xwidgets.XTextField;
import org.Xwidgets.XWidgetsException;
import org.apache.log4j.Logger;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;

import statics.StaticValues;

import jcifs.smb.SmbException;
import jcifs.smb.SmbFile;
import jcifs.smb.SmbFilenameFilter;

/**
 * Class offers a CIFS/SMB Networkbrowser
 */
class NetworkBrowser {

	private XFrame frame;
	protected XTextField adress;
	protected XTable table;
	private XLabel extensions;
	protected XButton close, select;
	protected XArrowButton oneUp;
	private WidgetFactory wt;
	protected SmbFile current;
	private String accepted[] = null;
	protected SmbFilenameFilter filter;
	private Object source = null, rows, rowsLayout;
	private Logger logger;
	
	/**
	 * Create a new NetworkBrowser
	 * 
	 * @param layout			Layout for the browser
	 * @param layoutData		layoutdata for use with the given layout
	 * @param fac				WidgetFactory
	 */
	protected NetworkBrowser(Object layout, Object layoutData, WidgetFactory fac){
		wt = fac;
		frame = wt.getFrame(MainGUI.mainGui.myFrame, -1, -1, Styles.RESIZE);
		logger = Logger.getLogger(getClass());
		try {
			frame.setLayout(layout);
			frame.setSize(600,650);
			frame.setText(StaticValues.getLangString("browseNetwork"));
			frame.closeOnClose(false);
			frame.setImage(MainGUI.mainGui.iconPrefix + "trayIcon.png");
			
			setLayout();
			
			XLabel header = wt.getLabel(frame, -1, -1, StaticValues.getLangString("browseNetwork").toUpperCase(),
					Styles.CENTER);
			header.setLayoutData(layoutData);
			header.setTooltipText(StaticValues.getLangString("networkBrowserHeader"));
			
			XPanel adressPanel = wt.getPanel(frame, -1, -1, Styles.BORDER);
			adressPanel.setLayout(rows);
			adressPanel.setLayoutData(rowsLayout);
			adress = wt.getTextField(adressPanel, -1, -1, "", Styles.BORDER|Styles.SINGLE);
			oneUp = wt.getArrowButton(adressPanel, -1, -1, Styles.UP);
			
			table = wt.getTable(frame, -1, -1, Styles.MULTI|Styles.BORDER, new String[]{StaticValues.getLangString("filename"), 
					StaticValues.getLangString("fileType")}, new int[]{400, 50});
			extensions = wt.getLabel(frame, -1, -1, "", Styles.SINGLE|Styles.BORDER);
			extensions.setLayoutData(layoutData);
			
			XPanel line = wt.getPanel(frame, -1, -1, Styles.SIMPLE);
			line.setLayout(rows);
			line.setLayoutData(rowsLayout);
			select = wt.getButton(line, -1, -1, StaticValues.getLangString("select"), Styles.NULL);
			close = wt.getButton(line, -1, -1, StaticValues.getLangString("close"), Styles.NULL);
			adress.setLayoutData(layoutData);
			adress.addKeyListener(KeyListener.getInstance());
			oneUp.addMouseListener(MouseListener.getInstance());
			
			table.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			close.setLayoutData(layoutData);
			select.setLayoutData(layoutData);
			table.addMouseListener(MouseListener.getInstance());
			close.addMouseListener(MouseListener.getInstance());
			select.addMouseListener(MouseListener.getInstance());
			
			filter = new SmbFilenameFilter(){
				public boolean accept(SmbFile arg0, String arg1) throws SmbException {
					try {
						SmbFile tmp = new SmbFile(arg0, arg1 + "/");
						if (tmp.isHidden()) return false;
						if (!tmp.isFile()) return true;
						if (accepted != null)
							for ( int i = 0; i < accepted.length; i++){
								if (tmp.getName().endsWith(accepted[i] + ((tmp.getName().endsWith("/")?"/":"")))) return true;
							}
						else return true;
					} catch (MalformedURLException e) {
						logger.debug(e.getMessage(), e);
					} catch (UnknownHostException e) {
						logger.debug(e.getMessage(), e);
					}
					return false;
				}
			};
		} catch (XWidgetsException e1) {
			logger.debug(e1.getMessage(), e1);
		}
	}
	
	/**
	 * set layoutdata for later use
	 *
	 */
	private void setLayout(){
		if (StaticValues.getSetting("gui").equals("swt")){
			rows = new GridLayout(2, false);
			rowsLayout = new GridData(GridData.FILL_HORIZONTAL);
			((GridData)rowsLayout).grabExcessHorizontalSpace = true;
			((GridData)rowsLayout).heightHint = 35;
		}
	}
	
	/**
	 * Source is needed to get the "caller"
	 * @return		caller
	 */
	public Object getSource(){
		return source;
	}
	
	/**
	 * Open the networkbrowser
	 * 
	 * @param caller			caller which is set to the source
	 * @param extensionName	name for allowed extensions
	 * @param fileExtensions	allowed extensions
	 */
	public void open(Object caller, String extensionName, String[] fileExtensions){
		try {
			source = caller;
			accepted = fileExtensions;
			extensionName += ":\t";
			for (int i = 0; i < accepted.length; i++)
				extensionName += "*." + accepted[i] + ((i < (accepted.length-1))?",":"");
			extensions.setText(extensionName);
			fillTable(new SmbFile("smb://"));
		} catch (MalformedURLException e) {
			logger.debug(e.getMessage(), e);
		}
		frame.setVisible(true);
	}
	
	/**
	 * close the frame but don't dispose
	 *
	 */
	public void close(){
		frame.setVisible(false);
	}

	/**
	 * update/fill the table
	 * 
	 * @param next		set a new currentobject
	 */
	protected void fillTable(SmbFile next){
		try {
			if (!next.isFile()){
				current = next;
				table.setEnabled(true);
				adress.setEnabled(true);
				select.setEnabled(true);
				oneUp.setEnabled(true);
				table.clear();
				adress.setText(current.toString().replaceFirst("smb:/", ""));
				String list[] = current.list(filter);
				for ( int i = 0; i < list.length; i++){
					SmbFile tmp = new SmbFile(current, list[i] + "/");
					String type = StaticValues.getLangString("file");
					if (tmp.getType() == SmbFile.TYPE_WORKGROUP) type = StaticValues.getLangString("workgroup");
					else if (tmp.getType() == SmbFile.TYPE_SERVER) type = StaticValues.getLangString("computer");
					else if (tmp.getType() == SmbFile.TYPE_SHARE) type = StaticValues.getLangString("share");
					else if (tmp.isDirectory()) type = StaticValues.getLangString("directory");
					table.addItem(new String[]{list[i], type});
				}
			}
		} catch (SmbException e) {
			logger.debug(e.getMessage(), e);
			table.clear();
			table.addItem(new String[]{StaticValues.getLangString("noNetwork"), ""});
			table.setEnabled(false);
			adress.setEnabled(false);
			select.setEnabled(false);
			oneUp.setEnabled(false);
		} catch (MalformedURLException e) {
			logger.debug(e.getMessage(), e);
		} catch (UnknownHostException e) {
			logger.debug(e.getMessage(), e);
		}
		
	}
}
