/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package gui;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XLabel;
import org.Xwidgets.XMenu;
import org.Xwidgets.XMenuItem;
import org.Xwidgets.XPanel;
import org.Xwidgets.XTable;
import org.Xwidgets.XWidgetsException;
import org.apache.log4j.Logger;
import org.jdom.Element;

import statics.StaticValues;

/**
 * PlaylistPanel offers the two panels panel and playlistsPanel. panel holds all elements for the current playlist,
 * playlistsPanel holds a table with possible playlists
 */
class PlaylistPanel {
	private static PlaylistPanel instance = null;
	protected XPanel panel, playlistsPanel;
	protected XLabel textCurrentPlaylist;
	protected XTable table, playlistTable;
	protected XMenu menu, menuPlayl;
	protected XMenuItem addFileToPl, addUrlToPl, addDirToPl, browseNetwork, delItemFromPl, clearPlaylist, showTitleProperties, shufflePlaylist, 
		delPlaylist, addPlaylist, loadPlaylistFromFile, renamePlaylist, openPlaylist, newPlaylist, savePlaylist;
	protected Object layout, layoutData;
	private Logger logger;
	private WidgetFactory wt;
	
	private PlaylistPanel(XPanel parent, Element panel, Object panelLayout, Object layoutData){
		try {
			logger = Logger.getLogger(getClass());
			wt = MainGUI.mainGui.myGUIFactory;
			layout = panelLayout;
			this.layoutData = layoutData;
			buildPlaylistPanel(parent, panel);
			buildPlaylistsPanel(parent, panel);
		}catch (Exception e){
			logger.debug(e.getMessage(), e);
		}
	}
	
	/**
	 * Get an instance of a PlaylistPanel
	 * 
	 * @param parent			The parent (maybe a dummy)
	 * @param panel			jdom Element
	 * @param panelLayout		layout
	 * @param layoutData		layoutdata
	 * @return					instance of PlaylistPanel
	 */
	protected static PlaylistPanel getPlaylistPanel(XPanel parent, Element panel, Object panelLayout, Object layoutData){
		if (instance == null)
			instance = new PlaylistPanel(parent, panel, panelLayout, layoutData);
		return instance;
	}

	/**
	 * Construct playlist panel
	 * 
	 * @param parent
	 * @param p
	 */
	private void buildPlaylistPanel(XPanel parent, Element p) {
		try {
			panel = wt.getPanel(parent, -1, -1, Styles.BORDER);
			panel.setLayout(layout);
			panel.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			
			XLabel header = wt.getLabel(panel, -1, -1, StaticValues.getLangString("playlist").toUpperCase(), 
					Styles.CENTER);
			header.setLayoutData(layoutData);
			header.setTooltipText(StaticValues.getLangString("playlistHeader"));
			
			table = wt.getTable(panel, -1, -1, Styles.BORDER|Styles.MULTI,
					new String[]{StaticValues.getLangString("id"),
				StaticValues.getLangString("title"),
				StaticValues.getLangString("artist"),
				StaticValues.getLangString("album")}, new int[]{45,400,120,200});
			table.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			table.addMouseListener(MouseListener.getInstance());
			table.addSelectionListener(SelectionListener.getInstance());
			
			menu = wt.getDropDownMenu(panel);
			addFileToPl = wt.getMenuItem(menu);
			addFileToPl.setText(StaticValues.getLangString("addFile"));
			addUrlToPl = wt.getMenuItem(menu);
			addUrlToPl.setText(StaticValues.getLangString("addURL"));
			addDirToPl = wt.getMenuItem(menu);
			addDirToPl.setText(StaticValues.getLangString("playlistAddDir"));
			browseNetwork = wt.getMenuItem(menu);
			browseNetwork.setText(StaticValues.getLangString("playlistBrowseNetwork"));
			delItemFromPl = wt.getMenuItem(menu);
			delItemFromPl.setText(StaticValues.getLangString("deletePlaylistEntry"));
			wt.getMenuSeparator(menu);
			clearPlaylist = wt.getMenuItem(menu);
			clearPlaylist.setText(StaticValues.getLangString("clear"));
			shufflePlaylist = wt.getMenuItem(menu);
			shufflePlaylist.setText(StaticValues.getLangString("shuffle"));
			savePlaylist = wt.getMenuItem(menu);
			savePlaylist.setText(StaticValues.getLangString("saveCurrentPlaylist"));
			wt.getMenuSeparator(menu);
			showTitleProperties = wt.getMenuItem(menu);
			showTitleProperties.setText(StaticValues.getLangString("showPlaylistEntry"));
			
			table.setMenu(menu);
			menu.addMenuListener(MenuListener.getInstance());
			addFileToPl.addSelectionListener(SelectionListener.getInstance());
			addUrlToPl.addSelectionListener(SelectionListener.getInstance());
			addDirToPl.addSelectionListener(SelectionListener.getInstance());
			browseNetwork.addSelectionListener(SelectionListener.getInstance());
			delItemFromPl.addSelectionListener(SelectionListener.getInstance());
			shufflePlaylist.addSelectionListener(SelectionListener.getInstance());
			clearPlaylist.addSelectionListener(SelectionListener.getInstance());
			savePlaylist.addSelectionListener(SelectionListener.getInstance());
			showTitleProperties.addSelectionListener(SelectionListener.getInstance());
		} catch (Exception e) {
			logger.debug(e.getMessage(), e);
		}
	}
	
	/**
	 * Build playlistPanel
	 * 
	 * @param parent
	 * @param p
	 */
	private void buildPlaylistsPanel(XPanel parent, Element p) {
		try {
			playlistsPanel = wt.getPanel(parent, -1, -1, Styles.BORDER);
			playlistsPanel.setLayout(layout);
			
			XLabel header = wt.getLabel(playlistsPanel, -1, -1, StaticValues.getLangString("playlists").toUpperCase(), 
					Styles.CENTER);
			header.setLayoutData(layoutData);
			header.setTooltipText(StaticValues.getLangString("playlistsHeader"));
			
			playlistTable = wt.getTable(playlistsPanel, -1, -1,
					Styles.BORDER|Styles.MULTI, new String[]{StaticValues.getLangString("id"),
					StaticValues.getLangString("playlist"),StaticValues.getLangString("items")}, new int[]{28,400,28});
			playlistTable.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			playlistTable.addMouseListener(MouseListener.getInstance());
			
			menuPlayl = wt.getDropDownMenu(playlistsPanel);
			
			loadPlaylistFromFile = wt.getMenuItem(menuPlayl);
			loadPlaylistFromFile.setText(StaticValues.getLangString("loadPlaylistFromFile"));
			newPlaylist = wt.getMenuItem(menuPlayl);
			newPlaylist.setText(StaticValues.getLangString("newMenu"));
			openPlaylist = wt.getMenuItem(menuPlayl);
			openPlaylist.setText(StaticValues.getLangString("openMenu"));
			addPlaylist = wt.getMenuItem(menuPlayl);
			addPlaylist.setText(StaticValues.getLangString("addPlaylistMenu"));
			renamePlaylist = wt.getMenuItem(menuPlayl);
			renamePlaylist.setText(StaticValues.getLangString("renameMenu"));
			delPlaylist = wt.getMenuItem(menuPlayl);
			delPlaylist.setText(StaticValues.getLangString("deleteMenu"));
			
			menuPlayl.addMenuListener(MenuListener.getInstance());
			playlistTable.setMenu(menuPlayl);
			openPlaylist.addSelectionListener(SelectionListener.getInstance());
			delPlaylist.addSelectionListener(SelectionListener.getInstance());
			renamePlaylist.addSelectionListener(SelectionListener.getInstance());
			addPlaylist.addSelectionListener(SelectionListener.getInstance());
			newPlaylist.addSelectionListener(SelectionListener.getInstance());
			loadPlaylistFromFile.addSelectionListener(SelectionListener.getInstance());
		} catch (XWidgetsException e) {
			logger.debug(e.getMessage(), e);
		}
		
	}
}
