/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package gui;

import java.util.Arrays;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XButton;
import org.Xwidgets.XFrame;
import org.Xwidgets.XLabel;
import org.Xwidgets.XPanel;
import org.Xwidgets.XTabbedPane;
import org.Xwidgets.XTable;
import org.Xwidgets.XWidgetsException;
import org.apache.log4j.Logger;

import core.Management;
import core.Playlist;
import core.Title;

import statics.StaticValues;

/**
 * PrefGUI offers a way to edit the configurations from back- and frontend
 */

class PrefGUI {
	protected XFrame myFrame;
	private WidgetFactory wt;
	
	protected XButton closeButton;
	protected XTable backendPrefs, frontendPrefs;
	protected XTabbedPane tabbed;
	private Logger logger;
	private Management myManagement;
	
	/**
	 * Create a new PrefGUI with a XTabbedPane and 2 entries, one for back- and one for frontend 
	 *
	 */
	protected PrefGUI() {
		logger = Logger.getLogger(getClass());
		myManagement = Management.getInstance();
		wt = MainGUI.mainGui.myGUIFactory;
		myFrame = wt.getFrame(null, 700, 600, Styles.RESIZE);
		myFrame.closeOnClose(false);
		myFrame.setImage(MainGUI.mainGui.iconPrefix + "trayPrefs.png");
		try {
			myFrame.setText(StaticValues.getLangString("prefGUI"));
			myFrame.setLayout(MainGUI.mainGui.layoutWithoutBorders);
		} catch (XWidgetsException e) {
			logger.debug(e.getMessage(), e);
		}
		buildMe();
	}
	
	/**
	 * update and open the window
	 *
	 */
	public void open(){
		update();
		tabbed.setSelection(0);
		myFrame.setVisible(true);
	}
	
	/**
	 * close the frame
	 *
	 */
	public void close(){
		myFrame.setVisible(false);
	}
	
	/**
	 * Construct content
	 *
	 */
	private void buildMe(){
		try {
			XLabel header = wt.getLabel(myFrame, -1, -1, StaticValues.getLangString("preferences").toUpperCase(), 
					Styles.CENTER);
			header.setTooltipText(StaticValues.getLangString("preferencesHeader"));
			header.setLayoutData(MainGUI.mainGui.layoutDataFillHoriz);
			
			tabbed = wt.getTabbedPane(myFrame, -1, -1, Styles.SIMPLE);
			tabbed.setLayout(MainGUI.mainGui.layoutWithoutBorders);
			tabbed.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			
			XPanel backend = wt.getPanel(myFrame, -1, -1, Styles.SIMPLE);
			backend.setLayout(MainGUI.mainGui.layoutWithBorders);
			backend.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			XPanel frontend = wt.getPanel(myFrame, -1, -1, Styles.SIMPLE);
			frontend.setLayout(MainGUI.mainGui.layoutWithBorders);
			frontend.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			
			backendPrefs = wt.getTable(backend, -1, -1, Styles.BORDER,
					new String[]{StaticValues.getLangString("prefKey"), StaticValues.getLangString("prefValue")}, 
					new int[]{300, 300});
			backendPrefs.addMouseListener(MouseListener.getInstance());
			backendPrefs.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			
			frontendPrefs = wt.getTable(frontend, -1, -1, Styles.BORDER,
					new String[]{StaticValues.getLangString("prefKey"), StaticValues.getLangString("prefValue")}, 
					new int[]{300, 300});
			frontendPrefs.addMouseListener(MouseListener.getInstance());
			frontendPrefs.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			
			closeButton = wt.getButton(myFrame, -1 , -1, 
					StaticValues.getLangString("ButtonClose"), Styles.CENTER);
			closeButton.addMouseListener(MouseListener.getInstance());
			closeButton.setLayoutData(MainGUI.mainGui.layoutDataFillHoriz);
			
			tabbed.addItem(backend, StaticValues.getLangString("backendprefs"), "");
			tabbed.addItem(frontend, StaticValues.getLangString("x4xprefs"), "");
		} catch (Throwable e){
			logger.debug(e.getMessage(), e);
		}
		update();
	}
	
	/**
	 * UpdateListener calls this method when a configval_update occurs
	 *
	 */
	protected void update() {
		String[] keys = myManagement.getConfigs();
		Arrays.sort(keys);
		int i = 0;
		for (i = 0; i < keys.length; i++){
			if (i < backendPrefs.getItemCount())
				backendPrefs.setText(new String[]{keys[i], myManagement.getConfigVal(keys[i])}, i);
			else backendPrefs.addItem(new String[]{keys[i], myManagement.getConfigVal(keys[i])});
		}
		backendPrefs.removeAllFrom(i);
		keys = StaticValues.getEditableKeys();
		Arrays.sort(keys);
		i = 0;
		for (i = 0; i < keys.length; i++){
			if (i < frontendPrefs.getItemCount())
				frontendPrefs.setText(new String[]{keys[i], StaticValues.getSetting(keys[i])}, i);
			else frontendPrefs.addItem(new String[]{keys[i], StaticValues.getSetting(keys[i])});
		}
		frontendPrefs.removeAllFrom(i);
	}
}
