/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package gui;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XButton;
import org.Xwidgets.XFrame;
import org.Xwidgets.XLabel;
import org.Xwidgets.XPanel;
import org.Xwidgets.XTextField;
import org.apache.log4j.Logger;

import core.Title;

import statics.StaticValues;

/**
 * class offers a window to edit titleproperties
 */

class PropGUI {
	private XFrame myFrame;
	private WidgetFactory wt;
	
	protected XLabel labelLength, labelFileType, labelBitRate, labelSampleRate, labelID;
	protected XTextField textTitle, textArtist, textAlbum, textDate, textComment, textGenre, textURL;
	protected XButton okButton, cancelButton;
	private XPanel propPanel, butPanel, detPanel;
	private Logger logger;
	private Title current;
	
	/**
	 * Build the PropGUI
	 */
	protected PropGUI() {
		logger = Logger.getLogger(getClass());
		
		wt = MainGUI.mainGui.myGUIFactory;
		myFrame = wt.getFrame(null, 450, 300, Styles.SIMPLE);
		myFrame.closeOnClose(false);
		myFrame.setImage(MainGUI.mainGui.iconPrefix + "trayIcon.png");
		buildPropPanel();
		buildButPanel();
		buildDetPanel();
		
		okButton.addMouseListener(MouseListener.getInstance());
		cancelButton.addMouseListener(MouseListener.getInstance());
		
		myFrame.setText(StaticValues.getLangString("propGUI"));
	}
	
	/**
	 * open the Window and set t as the current title
	 * @param t		Title to handle
	 */
	public void open(Title t){
		if ( t != null){
			current = t;
			myFrame.setVisible(true);
		}
	}
	
	/**
	 * Close the window and return the current Title
	 * 
	 * @return		current Title
	 */
	public Title close(){
		myFrame.setVisible(false);
		return current;
	}
	
	/**
	 * Construct editable content
	 *
	 */
	private void buildPropPanel(){
		propPanel = wt.getPanel(myFrame, 450,230, Styles.BORDER);
		wt.getLabel(propPanel, 70, 18, StaticValues.getLangString("title") + ":", Styles.LEFT).setLocation(30,20);
		wt.getLabel(propPanel, 70, 18, StaticValues.getLangString("artist") + ":", Styles.LEFT).setLocation(30,50);
		wt.getLabel(propPanel, 70, 18, StaticValues.getLangString("album") + ":", Styles.LEFT).setLocation(30,80);
		wt.getLabel(propPanel, 70, 18, StaticValues.getLangString("date") + ":", Styles.LEFT).setLocation(30, 110);
		wt.getLabel(propPanel, 70, 18, StaticValues.getLangString("comment") + ":", Styles.LEFT).setLocation(30,140);
		wt.getLabel(propPanel, 70, 18, StaticValues.getLangString("genre") + ":", Styles.LEFT).setLocation(30, 170);
		wt.getLabel(propPanel, 70, 18, StaticValues.getLangString("url") + ":", Styles.LEFT).setLocation(30, 200);
		
		textTitle = wt.getTextField(propPanel, 140, 22, "", Styles.LEFT|Styles.BORDER);
		textArtist = wt.getTextField(propPanel, 140, 22, "", Styles.LEFT|Styles.BORDER);
		textAlbum = wt.getTextField(propPanel, 140, 22, "", Styles.LEFT|Styles.BORDER);
		textDate = wt.getTextField(propPanel, 140, 22, "", Styles.LEFT|Styles.BORDER);
		textComment = wt.getTextField(propPanel, 140, 22, "", Styles.LEFT|Styles.BORDER);
		textGenre = wt.getTextField(propPanel, 140, 22, "", Styles.LEFT|Styles.BORDER);
		textURL = wt.getTextField(propPanel, 325, 22, "", Styles.LEFT|Styles.BORDER);
		
		textTitle.setLocation(105, 15);
		textArtist.setLocation(105, 45);
		textAlbum.setLocation(105, 75);
		textDate.setLocation(105, 105);
		textComment.setLocation(105, 135);
		textGenre.setLocation(105, 165);
		textURL.setLocation(105, 195);				
	}
	
	/**
	 * Construct non editable content
	 *
	 */
	private void buildDetPanel(){
		detPanel = wt.getPanel(propPanel, 190,180, Styles.LEFT);
		detPanel.setLocation(260,15);
		
		(labelLength = wt.getLabel(detPanel, 160, 22, StaticValues.getLangString("length") + ": 0:00", Styles.CENTER)).setLocation(20, 5);
		(labelFileType = wt.getLabel(detPanel, 160, 22, StaticValues.getLangString("fileType") + ": unknown", Styles.CENTER)).setLocation(20, 42);
		(labelBitRate = wt.getLabel(detPanel, 160, 22, StaticValues.getLangString("bitRate") + ": unknown", Styles.CENTER)).setLocation(20,79);
		(labelSampleRate = wt.getLabel(detPanel, 160, 22, StaticValues.getLangString("sampleRate") + ": unknown", Styles.CENTER)).setLocation(20,116);
		(labelID = wt.getLabel(detPanel, 160, 22, StaticValues.getLangString("id") + ": unknown", Styles.CENTER)).setLocation(20,155);
	}
	
	/**
	 * Construct Buttonpanel
	 *
	 */
	private void buildButPanel(){
		butPanel = wt.getPanel(myFrame, 450,70, Styles.BORDER);
		butPanel.setLocation(0,230);
		
		okButton = wt.getButton(butPanel, 70, 30, StaticValues.getLangString("ButtonOk"), Styles.CENTER);
		cancelButton = wt.getButton(butPanel, 70, 30, StaticValues.getLangString("ButtonCancel"), Styles.CENTER);
		
		okButton.setLocation(290, 2);
		cancelButton.setLocation(362, 2);
	}
}
