/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package gui;

import java.io.File;
import java.io.FileFilter;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.List;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XComboInputBox;
import org.Xwidgets.XDirectoryChooser;
import org.Xwidgets.XFileChooser;
import org.Xwidgets.XInputBox;
import org.Xwidgets.XInputValidator;
import org.Xwidgets.XMessageBox;
import org.Xwidgets.XTableItem;
import org.Xwidgets.event.XSelectionAdapter;
import org.Xwidgets.event.XSelectionEvent;
import org.apache.log4j.Logger;

import statics.StaticValues;

import core.Management;
import core.Title;

/**
 * Package SelectionListener
 */
class SelectionListener extends XSelectionAdapter{
	private Logger logger;
	private static SelectionListener instance;
	private Management myManagement;
	private WidgetFactory wt;
	protected static String chooserDir = StaticValues.getSetting("userhome");
	
	private SelectionListener(){
		logger = Logger.getLogger(getClass());
		myManagement = Management.getInstance();
		wt = MainGUI.mainGui.myGUIFactory;
	}
	
	/**
	 * Get a Listenerinstance
	 * 
	 * @return		SelectionListener instance
	 */
	public static SelectionListener getInstance(){
		if (instance == null)
			instance = new SelectionListener();
		return instance;
	}
	
	/**
	 * handle WidgetSelected event
	 */
	public void xWidgetSelected(XSelectionEvent e){
		int index[] = MainGUI.mainGui.playlistPanel.table.getSelectedIndices();
		if (MainGUI.mainGui.playlistPanel.table.indexOf(e.getSource()) != -1){
			int x = MainGUI.mainGui.playlistPanel.table.indexOf(e.getSource());
			switch (x){
				case 0:		myManagement.playlistSort("id"); break;
				case 1:		myManagement.playlistSort("title"); break;
				case 2:		myManagement.playlistSort("artist"); break;
				case 3:		myManagement.playlistSort("album"); break;
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.delItemFromPl)){
			if (index.length > 0){
				XMessageBox box = wt.getMessageBox(MainGUI.mainGui.myFrame, Styles.OK|Styles.CANCEL|Styles.QUESTION);
				box.setText(StaticValues.getLangString("reallyDeleteItems"));
				box.open();
				if (box.getButton() == Styles.OK){
					long indices[] = new long[index.length];
					for ( int i = 0; i < index.length; i++)
						indices[i] = index[i];
					myManagement.removeTitlesFromPlaylist(indices);
				}
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.mlibPanel.searchBox.getRealObject())){
			String entry = MlibPanel.getTextFromComboIndex(MainGUI.mainGui.mlibPanel.searchBox);
			MainGUI.mainGui.mlibPanel.preSearchbox.setTooltipText(StaticValues.getLangString("searchFor"));
			MainGUI.mainGui.mlibPanel.textSearch.setTooltipText(StaticValues.getLangString("searchFor"));
			MainGUI.mainGui.mlibPanel.textSearch.setText("");
			MainGUI.mainGui.mlibPanel.strict.setEnabled(true);
			if (entry.equals("all")){
				MainGUI.mainGui.mlibPanel.textSearch.setText("");
				MainGUI.mainGui.mlibPanel.textSearch.setEnabled(false);
				MainGUI.mainGui.mlibPanel.searchButton.setEnabled(false);
				MainGUI.mainGui.mlibPanel.strict.setEnabled(false);
				String[] query = new String[3];
				query[0] = "";
				query[1] = "";
				query[2] = "own";
				myManagement.mlibSelect(query, true);
			}
			else {
				MainGUI.mainGui.mlibPanel.textSearch.setEnabled(true);
				MainGUI.mainGui.mlibPanel.searchButton.setEnabled(true);
				MainGUI.mainGui.mlibPanel.strict.setEnabled(true);
				MainGUI.mainGui.mlibPanel.searchTable.clear();
			}
			if (entry.equals("ownQuery")){
				MainGUI.mainGui.mlibPanel.preSearchbox.setTooltipText(myManagement.getDefaultSQLString() + " ..");
				MainGUI.mainGui.mlibPanel.textSearch.setTooltipText(myManagement.getDefaultSQLString() + " ..");
				MainGUI.mainGui.mlibPanel.strict.setEnabled(false);
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.clearPlaylist)){
			XMessageBox box = wt.getMessageBox(MainGUI.mainGui.myFrame, Styles.OK|Styles.CANCEL|Styles.QUESTION);
			box.setText(StaticValues.getLangString("reallyClearPlaylist"));
			box.open();
			if (box.getButton() == Styles.OK)
				myManagement.playlistClear();
		}
		else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.showTitleProperties)){
			int items[] = MainGUI.mainGui.playlistPanel.table.getSelectedIndices();
			if (items.length > 0){
				buildPropGUI(myManagement.getTitleForIndex(items[0]));
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.mlibPanel.titleInfo)){
			int items[] = MainGUI.mainGui.mlibPanel.searchTable.getSelectedIndices();
			if (items.length > 0){
				try {
					buildPropGUI(myManagement.getTitleForID(Integer.parseInt
						(MainGUI.mainGui.mlibPanel.searchTable.getItemForIndex(items[0]).getText(0))));
				}catch (Throwable t){
					logger.debug(t.getMessage(), t);
				}
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.mlibPanel.browseNetwork)){
			MainGUI.mainGui.CIFSBrowser.open(MainGUI.mainGui.mlibPanel.browseNetwork,
					StaticValues.getLangString("mediafiles"), myManagement.getFileTypes());
		}
		else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.browseNetwork)){
			MainGUI.mainGui.CIFSBrowser.open(MainGUI.mainGui.playlistPanel.browseNetwork, 
					StaticValues.getLangString("mediafiles"), myManagement.getFileTypes());
		}
		else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.savePlaylist)){
			XInputBox in = wt.getInputBox(null, Styles.QUESTION);
			in.setName(StaticValues.getLangString("savePlaylistAs"));
			in.setText(StaticValues.getLangString("savePlaylistAs"));
			in.setInputValidator(new XInputValidator(){
				public String isValid(String text) {
					if (text.startsWith("_")) return StaticValues.getLangString("noUnderscores");
					if (text.length() < 3) return StaticValues.getLangString("tooShort");
					return null;
				}
			});
			in.open();
			if (in.getButton() == Styles.OK){
				myManagement.mlibSaveCurrentPlaylist(in.getInput());
				myManagement.mlibGetPlaylists();
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.shufflePlaylist)) 
			myManagement.playlistShuffle();
		else if (e.getSource().equals(MainGUI.mainGui.menuExit) 
				|| e.getSource().equals(MainGUI.mainGui.trayExit))
			MainGUI.mainGui.finalizer();
		else if (e.getSource().equals(MainGUI.mainGui.menuPrefs))
			MainGUI.mainGui.prefGui.open();
		else if (MainGUI.mainGui.lists != null && e.getSource().equals(MainGUI.mainGui.menuMlib)){
			if (MainGUI.mainGui.tabbedPane != null)
				MainGUI.mainGui.tabbedPane.setSelection(0);
			MainGUI.mainGui.lists.setVisible(true);
		}
		else if (e.getSource().equals(MainGUI.mainGui.aboutEntry) 
				|| e.getSource().equals(MainGUI.mainGui.trayAbout))
			new AboutWindow();
		else if (e.getSource().equals(MainGUI.mainGui.trayPlay))
			myManagement.play();
		else if (e.getSource().equals(MainGUI.mainGui.trayStop))
			myManagement.stop();
		else if (e.getSource().equals(MainGUI.mainGui.trayPause))
			myManagement.pause();
		else if (e.getSource().equals(MainGUI.mainGui.trayNext))
			myManagement.next();
		else if (e.getSource().equals(MainGUI.mainGui.trayPrev))
			myManagement.prev();
		else if (e.getSource().equals(MainGUI.mainGui.helpEntry)){
			XMessageBox box = wt.getMessageBox(MainGUI.mainGui.myFrame, Styles.OK);
			box.setText(StaticValues.getLangString("noHelpAtm"));
			box.open();
		}
		else if (e.getSource().equals(MainGUI.mainGui.eqEntry)){
			MainGUI.mainGui.buildEQ();
			MainGUI.mainGui.eqWindow.setVisible(true);
		}
		else if (e.getSource().equals(MainGUI.mainGui.mlibPanel.browseBox.getRealObject())){
			String[] query = new String[3];
			query[0] = "Media WHERE key='" + MlibPanel.getTextFromComboIndex(MainGUI.mainGui.mlibPanel.browseBox) + "'";
			query[1] = "";
			query[2] = "own";
			myManagement.mlibSelect(query, false);
		}
		else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.loadPlaylistFromFile)){
			XFileChooser xfc = wt.getFileChooser(MainGUI.mainGui.
					myFrame, Styles.FD_OPEN|Styles.SINGLE);
			xfc.setSelectedPath(chooserDir);
			xfc.addFileFilter(StaticValues.getLangString("playlists"), myManagement.getPlaylistTypes());
			xfc.open();
			chooserDir = xfc.getSelectedPath();
			String[] selected = xfc.getSelectedFiles();
			if (selected.length > 0 && !selected[0].equals("")){
				String complete = xfc.getSelectedPath() + File.separator + selected[0];
				myManagement.mlibLoadPlaylist("file://" + complete, 
						selected[0].substring(0,selected[0].indexOf(".")));
			}
			myManagement.mlibGetPlaylists();
		}
		else if (e.getSource().equals(MainGUI.mainGui.mlibPanel.searchFor)){
			int items[] = MainGUI.mainGui.mlibPanel.browseTable.getSelectedIndices();
			if (items.length > 0){
				String[] query = new String[3];
				query[0] = MlibPanel.getTextFromComboIndex(MainGUI.mainGui.mlibPanel.browseBox);
				query[1] = MainGUI.mainGui.mlibPanel.browseTable.getItemForIndex(items[0]).getText(0);
				query[2] = "exact";	
				myManagement.mlibSelect(query, true);
				MainGUI.mainGui.mlibPanel.textSearch.setText(query[1]);
				MainGUI.mainGui.mlibPanel.strict.setChecked(true);
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.mlibPanel.addEntryMedialib)){
			String[] selected = readFilesViaChooser();
			if (selected.length > 0){
				List l = new ArrayList();
				for (int i = 0; i < selected.length; i++){
					if (!isPlaylistType(selected[i]))
						l.add("file://" + selected[i]);
					else {
						myManagement.mlibLoadPlaylist("file://" + selected[i], 
								selected[i].substring(selected[i].lastIndexOf("/")+1,selected[i].indexOf(".")));
					}
				}
				myManagement.addURLListToMedialib(l);
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.mlibPanel.addRandomEntries)){
			XComboInputBox box = wt.getComboInputBox(MainGUI.mainGui.myFrame, Styles.READ_ONLY);
			box.setText(StaticValues.getLangString("mlibAddRandom"));
			box.setName(StaticValues.getLangString("mlibAddRandom"));
			String array[] = new String[300];
			for ( int i = 0; i < array.length; i++)
				array[i] = ""+(i+1);
			box.setData(array);
			box.setIndex(19);
			box.open();
			if (box.getButton() == Styles.OK && box.getIndex() > 0){
				String query = " WHERE key='url' ORDER BY RANDOM() LIMIT " + box.getIndex();
				myManagement.mlibAddQueryToPlaylist(query);
				if (MainGUI.mainGui.tabbedPane != null)
					MainGUI.mainGui.tabbedPane.setSelection(MainGUI.mainGui.
							tabbedPane.getIndex(MainGUI.mainGui.playlistPanel.panel));
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.mlibPanel.addDirectory)){
			XDirectoryChooser xfc = wt.getDirectoryChooser(MainGUI.mainGui.myFrame);
			xfc.setSelectedPath(chooserDir);
			xfc.open();
			chooserDir = xfc.getSelectedPath();
			logger.debug("chosen path: " + xfc.getSelectedPath());
			if (!xfc.getSelectedPath().equals("")){
				List l = new ArrayList();
				recurseDir(new File(xfc.getSelectedPath()), l);
				myManagement.addURLListToMedialib(l);
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.addDirToPl)){
			XDirectoryChooser xfc = wt.getDirectoryChooser(MainGUI.mainGui.myFrame);
			xfc.setSelectedPath(chooserDir);
			xfc.open();
			chooserDir = xfc.getSelectedPath();
			logger.debug("chosen path: " + xfc.getSelectedPath());
			if (!xfc.getSelectedPath().equals("")){
				List l = new ArrayList();
				recurseDir(new File(xfc.getSelectedPath()), l);
				myManagement.addURLListToPlaylist(l);
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.mlibPanel.refreshMlib)){
			myManagement.mlibRefresh();
		}
		else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.addFileToPl)){
			String[] selected = readFilesViaChooser();
			if (selected.length > 0)
				for (int i = 0; i < selected.length; i++){
					if (!isPlaylistType(selected[i]))
						myManagement.addTitleToPlaylist("file://" + selected[i]);
					else {
						myManagement.mlibLoadPlaylist("file://" + selected[i], 
								selected[i].substring(selected[i].lastIndexOf("/")+1,selected[i].indexOf(".")));
						myManagement.mlibAddPlaylistToCurrent(
								selected[i].substring(selected[i].lastIndexOf("/")+1,selected[i].indexOf(".")));
					}
				}
		}
		else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.addUrlToPl)){
			XInputBox in = wt.getInputBox(null, Styles.QUESTION);
			in.setName(StaticValues.getLangString("addURL"));
			in.setText(StaticValues.getLangString("addURL"));
			in.open();
			if (in.getButton() == Styles.OK){
				String text = in.getInput();
				myManagement.addTitleToPlaylist(text);
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.mlibPanel.addToPlaylist)) {
			int items[] = MainGUI.mainGui.mlibPanel.searchTable.getSelectedIndices();
			if (items.length > 0){
				for ( int i = 0; i < items.length; i++){
					myManagement.addIdToPlaylist(new Integer(MainGUI.mainGui.mlibPanel.searchTable.getItemForIndex
							(items[i]).getText(0)).intValue());
					logger.debug("id: " + new Integer(MainGUI.mainGui.mlibPanel.searchTable.getItemForIndex
							(items[i]).getText(0)).intValue());
				}
				if (MainGUI.mainGui.tabbedPane != null)
					MainGUI.mainGui.tabbedPane.setSelection(MainGUI.mainGui.tabbedPane.getIndex(MainGUI.mainGui.playlistPanel.panel));
			}
		}
		else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.newPlaylist)) {
			XInputBox newName = wt.getInputBox(null, Styles.QUESTION);
			newName.setInput(StaticValues.getLangString("newPlaylistName"));
			newName.setName((StaticValues.getLangString("enterNewPlaylistName")));
			newName.setText((StaticValues.getLangString("enterNewPlaylistName")));
			newName.setInputValidator(new XInputValidator(){
				public String isValid(String text) {
					if (text.startsWith("_")) return StaticValues.getLangString("noUnderscores");
					if (text.length() < 3) return StaticValues.getLangString("tooShort");
					return null;
				}
			});
			newName.open();
			
			if (newName.getButton() == Styles.OK){
				String name = newName.getInput();
				myManagement.mlibNewPlaylist(name);
				myManagement.mlibLoadPlaylist(name);
				myManagement.mlibGetPlaylists();
			}
		} // end new
		
		// playlist
		else if (MainGUI.mainGui.playlistPanel.playlistTable.getItemCount() > 0) {
			int items[] = MainGUI.mainGui.playlistPanel.playlistTable.getSelectedIndices();
			if (e.getSource().equals(MainGUI.mainGui.playlistPanel.openPlaylist)) {
				if (items.length > 0) {
					myManagement.mlibLoadPlaylist(MainGUI.mainGui.playlistPanel.playlistTable.
							getItemForIndex(items[0]).getText(1));
					if (MainGUI.mainGui.tabbedPane != null)
						MainGUI.mainGui.tabbedPane.setSelection(MainGUI.mainGui.tabbedPane.getIndex(MainGUI.mainGui.playlistPanel.panel));
				}
			}
			
			else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.addPlaylist)) {
				if (items.length > 0){
					myManagement.mlibAddPlaylistToCurrent(MainGUI.mainGui.playlistPanel.playlistTable.
						getItemForIndex(items[0]).getText(1));
					if (MainGUI.mainGui.tabbedPane != null)
						MainGUI.mainGui.tabbedPane.setSelection(MainGUI.mainGui.
								tabbedPane.getIndex(MainGUI.mainGui.playlistPanel.panel));
				}
			}
			
			else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.delPlaylist)) {
				if ( items.length > 0){
					XMessageBox question = wt.getMessageBox(MainGUI.mainGui.myFrame, Styles.OK|Styles.CANCEL|Styles.QUESTION);
					question.setText(StaticValues.getLangString("reallyDeletePlaylist"));
					question.open();
					if (question.getButton() == Styles.OK){
						for ( int i = 0; i < items.length; i++){
							myManagement.mlibRemovePlaylist(MainGUI.mainGui.playlistPanel.
									playlistTable.getItemForIndex(items[i]).getText(1));
						}
						myManagement.mlibGetPlaylists();
					}
				}
			}
			
			else if (e.getSource().equals(MainGUI.mainGui.playlistPanel.renamePlaylist)) {
					if (items.length > 0) {
						MainGUI.mainGui.playlistPanel.playlistTable.setSelectedIndex(items[0]);
						XTableItem xti = MainGUI.mainGui.playlistPanel.playlistTable.getItemForIndex(items[0]);
						String nameold = xti.getText(1);
						
						if (xti != null){
							XInputBox newName = wt.getInputBox(null, Styles.QUESTION);
							newName.setInput(nameold);
							newName.setName((StaticValues.getLangString("renamePlaylist")));
							newName.setText((StaticValues.getLangString("renamePlaylist")));
							newName.setInputValidator(new XInputValidator(){
								public String isValid(String text) {
									if (text.startsWith("_")) return StaticValues.getLangString("noUnderscores");
									if (text.length() < 3) return StaticValues.getLangString("tooShort");
									return null;
								}
							});
							newName.open();
							
							if (newName.getButton() == Styles.OK){
								String name = newName.getInput();
								myManagement.mlibRenamePlaylist(nameold,name);
								xti.setText(1, name);
							}			
						} 
					}
			} // end rename
		}
	}

	/**
	 * Recurse over a given directory and fill the List
	 * 
	 * @param file
	 * @param l
	 */
	private void recurseDir(File file, List l) {
		if (file.exists() && file.isDirectory()){
			File list[] = file.listFiles(new FileFilter(){
				public boolean accept(File arg0) {
					if (arg0.isDirectory()) return true;
					String supported[] = myManagement.getFileTypes();
					for ( int i = 0; i < supported.length; i++)
						if (arg0.getName().endsWith(supported[i]))
							return true;
					return false;
				}
			});
			for ( int i = 0; i < list.length; i++){
				if (list[i].isDirectory())
					recurseDir(list[i], l);
				else l.add("file://" + list[i]);
			}
		}
	}
	
	/**
	 * check if the given string ends with a playlist-type
	 * 
	 * @param name
	 * @return		true if playlist, false otherwise
	 */
	private boolean isPlaylistType(String name){
		String playlists[] = myManagement.getPlaylistTypes();
		for (int i = 0; i < playlists.length; i++)
			if (name.endsWith(playlists[i]))return true;
		return false;
	}

	/**
	 * Read files via a XFileChooser
	 * 
	 * @return		read elements as String with path and name
	 */
	private String[] readFilesViaChooser() {
		XFileChooser xfc = wt.getFileChooser(MainGUI.mainGui.
				myFrame, Styles.FD_OPEN|Styles.MULTI);
		xfc.setSelectedPath(chooserDir);
		xfc.addFileFilter(StaticValues.getLangString("mediafiles"), myManagement.getAllSupportedTypes());
		xfc.open();
		String[] selected = xfc.getSelectedFiles();
		chooserDir = xfc.getSelectedPath();
		for (int i = 0; i < selected.length; i++)
			selected[i] = xfc.getSelectedPath() + File.separator + selected[i];
		return selected;
	}
	
	/**
	 * Fill Property gui with title information from Title t, set t as current and open the frame
	 * 
	 * @param t
	 */
	private void buildPropGUI(Title t) {
		int min, sec;
		
		if (t != null && (MainGUI.mainGui.propGui != null)) {
			try {
				MainGUI.mainGui.propGui.textTitle.setText(t.getAttribute("title"));
				MainGUI.mainGui.propGui.textArtist.setText(t.getAttribute("artist"));
				MainGUI.mainGui.propGui.textAlbum.setText(t.getAttribute("album"));
				MainGUI.mainGui.propGui.textGenre.setText(t.getAttribute("genre"));
				MainGUI.mainGui.propGui.textURL.setText(t.getAttribute("url"));
				MainGUI.mainGui.propGui.textDate.setText(t.getAttribute("date"));
				MainGUI.mainGui.propGui.textComment.setText(t.getAttribute("comment"));
				MainGUI.mainGui.propGui.labelFileType.setText(StaticValues.getLangString("fileType") + ": " + t.getAttribute("mime"));
				
				MainGUI.mainGui.propGui.labelID.setText("ID: " + t.getAttribute("id"));
				min = t.getLength()/60;
				sec = t.getLength()%60;
				MainGUI.mainGui.propGui.labelLength.setText(StaticValues.getLangString("length") + ": " + min + ":" + ((sec<10)?"0":"") + sec);		
				MainGUI.mainGui.propGui.labelBitRate.setText(StaticValues.getLangString("bitRate") + ": " + Integer.parseInt(t.getAttribute("bitrate"))/1000 + " kbit/s");
				MainGUI.mainGui.propGui.labelSampleRate.setText(StaticValues.getLangString("sampleRate") + ": " + t.getAttribute("samplerate") + " Hz");
			} catch (Exception e){
				logger.debug(e.getMessage(), e);
			}
			finally {
				MainGUI.mainGui.propGui.open(t);
			}
		}
	}
	
	/**
	 * Handle DefaultWidgetSelected event
	 */
	public void xWidgetDefaultSelected(XSelectionEvent arg0) {}
}
