/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************/

package statics;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;

/**
 * Class is used to hold frontendconfigs and languagestrings
 */
public final class StaticValues {
	private static ResourceBundle langStrings = ResourceBundle.getBundle("properties.lang.x4x", Locale.getDefault());
	private static Properties staticSettings = new Properties();
	private static Logger logger = Logger.getLogger(StaticValues.class);
	private static File config;
	
	public static void init(String configFile){
		staticSettings.setProperty("deli", System.getProperty("file.separator"));
		staticSettings.setProperty("userhome", System.getProperty("user.home"));
		staticSettings.setProperty("basedir", System.getProperty("user.dir"));
		staticSettings.setProperty("backend", "xmms2");
		staticSettings.setProperty("gui", "swt");
		staticSettings.setProperty("style", "default");
		staticSettings.setProperty("iconstyle", "default");
		staticSettings.setProperty("serverport", "");
		staticSettings.setProperty("serverip", "");
		staticSettings.setProperty("prefdir", System.getProperty("user.home") + staticSettings.get("deli") + 
				".xmms2" + staticSettings.get("deli") + "clients" + staticSettings.get("deli") + "x4x");
		staticSettings.setProperty("icondir", "" + staticSettings.get("prefdir") + staticSettings.get("deli") + 
				"images" + staticSettings.get("deli") + "icons" + staticSettings.get("deli"));
		staticSettings.setProperty("styledir", StaticValues.getSetting("prefDir") + StaticValues.getSetting("deli") + 
				"styles");
		try {
			config = new File(staticSettings.getProperty("prefdir") + 
					staticSettings.getProperty("deli") + "settings.properties");
			if (configFile != null && !configFile.equals("")){
				File f = new File(configFile);
				if (f.exists() && f.canRead())
					config = f;
			}
			staticSettings.load(new FileInputStream(config));
		} catch (FileNotFoundException e) {
			logger.debug(getLangString("noConfigFile"), e);
		} catch (IOException e) {
			logger.debug(getLangString("problemReadingConfigFile"), e);
		}
	}
	
	/**
	 * get a i18n string
	 * @param name
	 * @return			i18n String if there, otherwise ""
	 */
	public static String getLangString(String name){
		try {
			return langStrings.getString(name);
		}catch (Exception e){
			logger.debug("Unable to find matching String for key " + name);
			return "";
		}
	}
	
	/**
	 * @param name
	 * @return			get specific setting, "" if not there
	 */
	public static String getSetting(String name){
		try {
			String t = staticSettings.getProperty(name.toLowerCase());
			if ( t == null){
				logger.debug("Unable to find matching Setting for key " + name);
				t = "";
			}
			return t;
		}catch (Exception e){
			return "";
		}
	}
	
	/**
	 * @return		all editable keys from configs
	 */
	public static String[] getEditableKeys(){
		ArrayList keys = new ArrayList();
		for (Iterator it = staticSettings.keySet().iterator(); it.hasNext();){
			String next = ""+it.next();
			if (!next.equalsIgnoreCase("deli") && !next.equalsIgnoreCase("userhome") &&
					!next.equalsIgnoreCase("prefdir"))
				keys.add(next);
		}
		Collections.sort(keys);
		return (String[])keys.toArray(new String[]{});
	}
	
	/**
	 * @return		all keys from configs
	 */
	public static String[] getKeys(){
		String[] keys = new String[staticSettings.size()];
		int i = 0;
		for (Iterator it = staticSettings.keySet().iterator(); it.hasNext();)
			keys[i++] = ""+it.next();
		return keys;
	}
	
	/**
	 * Set a setting to a new value. Settings are only saved on shutdown
	 * @param key
	 * @param value
	 */
	public static void setSetting(String key, String value){
		staticSettings.setProperty(key.toLowerCase(), value);
	}

	/**
	 * write configs and shutdown
	 *
	 */
	public static void finish() {
		Properties tmp = (Properties)staticSettings.clone();
		tmp.remove("deli");
		tmp.remove("prefdir");
		tmp.remove("userhome");
		try {
			if (!config.getParentFile().exists())
				config.getParentFile().mkdirs();
			if (!config.exists())
				config.createNewFile();
			tmp.store(new FileOutputStream(config), "Don't change this file unless you know what you are doing!");
		} catch (FileNotFoundException e) {
		} catch (IOException e) {
			logger.debug(getLangString("cannotWriteConfig"));
		}
	}
}
