/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * Copyright Georg Schild (dangertools@gmail.com), Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package tests;

import java.util.LinkedList;
import java.util.List;

import core.KeepStateThread;
import core.Management;
import core.Playlist;
import core.ReturnValue;
import core.Title;
import core.UpdateUIType;
import junit.framework.TestCase;

public class PlayTests extends TestCase{
	private Management m = AllTests.m;
	private ReturnValue ret;
	private Playlist currentPl;
	private int timeout = 1000;
	
	protected void setUp() throws Exception {
		addTestTitles(); /* add some titles for testing */
	}
	
	protected void tearDown(){
		m.waitForID(m.stop()); /* stop the playback */
	}
	
	public void testPlay(){
		m.waitForID(m.stop()); /* stop the playback - if we were already in state playing*/
		m.waitForID(m.play()); /* start playback */
		
		try {
			Thread.sleep(timeout);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		currentPl = m.getCurrentPlaylist();
		if(currentPl.getSize() > 0)
			assertEquals(KeepStateThread.PLAYING, m.getStatus());
	}
	
	public void testStop(){
		setPlaying();
		m.waitForID(m.stop());
		
		try {
			Thread.sleep(timeout);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		currentPl = m.getCurrentPlaylist();
		if(currentPl.getSize() > 0)		
			assertEquals(KeepStateThread.IDLE, m.getStatus());
	}
	
	public void testPause(){
		setPlaying();
		m.waitForID(m.pause());
		try {
			Thread.sleep(timeout);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		currentPl = m.getCurrentPlaylist();
		if(currentPl.getSize() > 0)
			assertEquals(KeepStateThread.PAUSED, m.getStatus());
	}
	
	public void testNext(){
		int index;
		
		setPlaying();
		
		index = m.getPlaylistIndex();
		m.waitForID(m.next());
		
		currentPl = m.getCurrentPlaylist();
		
		if(index == (currentPl.getSize()-1))
			index--;
		
		try {
			Thread.sleep(timeout);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		
		if(currentPl.getSize() > 0)
			assertEquals(index+1, m.getPlaylistIndex());
	}
	
	public void testPrev(){
		int index;
		
		setPlaying();
		
		index = m.getPlaylistIndex();
		m.waitForID(m.prev());
		
		
		if(index == 0)
			index++;
		
		try {
			Thread.sleep(timeout);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		assertEquals(index-1, m.getPlaylistIndex());
	}
	
	public void testPlaylistIndex(){
		int index, n;
		n = 3;
		
		setPlaying();
			
		currentPl = m.getCurrentPlaylist();
		index = m.getPlaylistIndex();
		m.waitForID(m.nextInPlaylistAbs(n));
		
		try {
			Thread.sleep(timeout);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		if((index +n ) < currentPl.getSize())
			assertEquals((index+n), m.getPlaylistIndex());
		else
			assertEquals(index, m.getPlaylistIndex());
		
	}
	
	public void testPlaylistClear(){
		m.waitForID(m.playlistClear());
		List l = addTestTitles();
		try {Thread.sleep(timeout);} catch (InterruptedException e) {fail();}
		assertEquals(m.getCurrentPlaylist().getSize(), l.size());
		m.waitForID(m.playlistClear());
		try {Thread.sleep(timeout);} catch (InterruptedException e) {fail();}
		assertEquals(m.getCurrentPlaylist().getSize(), 0);
	}

	private void setPlaying() {
		if (m.getStatus() != KeepStateThread.PLAYING)
			m.waitForID(m.play());
	}
	
	private LinkedList addTestTitles() {
		LinkedList addedTitles = new LinkedList();
		LinkedList entries;

		ret = m.waitForID(m.mlibSelect(new String[]{"WHERE key='artist' AND value LIKE '%'","", "own"}, true));
		assertEquals(ReturnValue.MEDIALIB_SEARCH_TYPE,ret.getType());
		
		UpdateUIType uit = new UpdateUIType(UpdateUIType.MLIB_SEARCH, ret.getValue());
		entries = (LinkedList)uit.getVal();
		for (int i=0; (i < 10) && ( i < entries.size()); i++){
			Title t = (Title)entries.get(i);
			if (!addedTitles.contains(t)) {
				ret = m.waitForID(m.addIdToPlaylist(Integer.parseInt(t.getAttribute("id"))));
				assertEquals(ret.getType(), ReturnValue.VOID_TYPE);
				addedTitles.add(t);
			}
		}
		
		return addedTitles;
	}
}
