/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * Copyright Georg Schild (dangertools@gmail.com), Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package gui.models;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.ListIterator;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import statics.StaticValues;

public class DNA {
	private Logger logger;
	private String stylename = StaticValues.getSetting("style");
	private String styleDir = StaticValues.getSetting("styledir") +  StaticValues.getSetting("deli");
	private Document dnaDoc;
	private int maxDepth = 4;
	
	/**
	 * Create DNA - means, read style etc and offer its content
	 *
	 */
	public DNA(){
		logger = Logger.getLogger(getClass());
		logger.debug(StaticValues.getLangString("printStyleName") + " " + stylename);
		File styleFile = new File(styleDir + stylename + ".xml");
		if (!styleFile.exists()){
			stylename = "default";
			styleFile = new File(styleDir + stylename + ".xml");
		}
		if (styleFile.exists()){
			try {
				SAXBuilder builder = new SAXBuilder();
				dnaDoc = builder.build(styleFile);
			} catch (IOException e){
				logger.info(StaticValues.getLangString("noStyleFile"));
				logger.debug(e.getMessage(), e);
			} catch (JDOMException e){
				logger.info(StaticValues.getLangString("xmlError"));
				logger.debug(e.getMessage(), e);
			}
		}
	}
	
	/**
	 * get a list of children of a given Element
	 * 
	 * @param el
	 * @return		children
	 */
	public List getChildren(Element el){
		return el.getChildren();
	}
	
	/**
	 * get baseelement with given name
	 * 
	 * @param name
	 * @return			base element if found, null otherwise
	 */
	public Element getBaseElement(String name){
		try {
			return getChild(dnaDoc.getRootElement(), name, maxDepth-1);
		} catch (NullPointerException e){ return null; }
	}
	
	/**
	 * get child by name
	 * 
	 * @param node
	 * @param name
	 * @return			child with name name if found, null otherwise
	 */
	public Element getChild(Element node, String name){
		return getChild(node, name, 1);
	}
	
	/**
	 * search for child till depth depth
	 * 
	 * @param node
	 * @param value
	 * @param depth
	 * @return			child if found, null otherwise
	 */
	private Element getChild(Element node, String value, int depth){
		Element match = node.getChild(value);
		if (match == null && depth < maxDepth){
			List all = node.getChildren();
			ListIterator it = all.listIterator();
			while(it.hasNext()){
				Element next = (Element)it.next();
				Element ret = getChild(next, value, depth++);
				if (ret != null)
					return ret;
			}
		}
		return match;
	}
}
